#ifndef _DSND
#define _DSND

#include <windows.h>
#include <dsound.h>
#include "sptypes.h"

extern 	LPDIRECTSOUND 	dSound;		// Global object to sound device

inline void killDirectSound(void)
{
   if (dSound)
   {
      dSound->Release();
	  dSound = NULL;
   }
}

inline BOOL /*__fastcall*/ initDirectSound(HWND hwnd) // not done even 5Hz!!
{
   if (DirectSoundCreate(NULL,&dSound,NULL)==DS_OK)
   {
      dSound->SetCooperativeLevel(hwnd,DSSCL_NORMAL );
      return TRUE;
   }
   return FALSE;
}

BOOL createDSB(LPDIRECTSOUNDBUFFER *,WAVEFORMATEX *,DWORD);

class DSoundBuffer {
	
	LPDIRECTSOUNDBUFFER 	snd;
	DWORD 					writePtr;
	DWORD 					bufferSize;

	DWORD __fastcall UpdateWritePointer(DWORD x)
	{
   	   DWORD oldPtr = writePtr;
   	   DWORD bsize = GetSize();
   	   writePtr += x;
       if (writePtr >= bsize)
          writePtr -= bsize;
	   return oldPtr;
	}

public:

	BOOL __fastcall WriteBufferAvail(DWORD dataSize)
	{
		if (DonePlaying())
			return TRUE;
		if (PlayPointerInside(writePtr,dataSize)==FALSE)
			return TRUE;
		else
   			return FALSE;
	}

	DSoundBuffer();
	DSoundBuffer(WAVEFORMATEX *pcmwf,DWORD size);
	~DSoundBuffer();

	BOOL	__fastcall InitFromRAV(char *filename,DWORD size);
	BOOL	Open(WAVEFORMATEX *pcmwf ,DWORD size=0);
	void	Close(void);
	BOOL	ClearBuffer(DWORD size=0,BYTE value=128);
	
	BOOL	__fastcall GetFormat(WAVEFORMATEX *pcmwf)
	{
		DWORD bytes;
		if (snd->GetFormat(pcmwf, sizeof(*pcmwf), &bytes)==DS_OK)
			return TRUE;
		else
			return FALSE;
	}

	DWORD GetPlayPosition(void)
	{
    	DWORD 	cp,cw;
    	if (snd->GetCurrentPosition(&cp,&cw)==DS_OK)
			return cp;
		else
			return INVALID_VALUE;
	}

	void Reset(void) 
	{ 
	   writePtr=0;
	   Stop();
	   snd->SetCurrentPosition(0);
	}

	BOOL DonePlaying(void)
	{
		DWORD status;
	    if (snd)
		{
			snd->GetStatus(&status);
			if ( (status & DSBSTATUS_PLAYING) == DSBSTATUS_PLAYING)
				return FALSE;
		}
		return TRUE;
	}

    BOOL 	__fastcall PlayPointerInside(DWORD seg1,DWORD range)
	{
		DWORD pos = GetPlayPosition();
		DWORD seg2 = seg1 + range;
		if (seg2 > bufferSize)
		{
			seg2 -= bufferSize; // Range wrapped
			if (pos >= seg1 || pos < seg2)
	  			return TRUE;
		} else {
			if (pos >= seg1 && pos < seg2)
      			return TRUE;
		}
		return FALSE;
	}

	void	__fastcall SetPlayPosition(DWORD p) { snd->SetCurrentPosition(p); }
	HRESULT __fastcall Play(int flag=DSBPLAY_LOOPING) { return snd->Play(0,0,flag); }
	HRESULT Stop(void) { return snd->Stop(); }

	void 	__fastcall SetVolume(LONG volume) 
	{ 
		snd->SetVolume(volume); 
	}
	void 	__fastcall SetPanPosition(LONG pan) { snd->SetPan(pan); }
	LONG 	GetVolume(void) { LONG vol; snd->GetVolume(&vol); return vol; }
	LONG 	GetPanPosition(void) { LONG pan; snd->GetPan(&pan); return pan; }

	DWORD 	GetSize(void) { return bufferSize; }
    DWORD   __fastcall FillSoundBuffer(LPVOID ptr,DWORD size);
};

#endif
