/***************************************************************************\
						File Resource Module
\***************************************************************************/

#include "FRsrc.h"
#include "dbgtst.h"

// Return byte per second we can read
int __fastcall FRSRC::Calibrate(DWORD secs, long chunkSize)
{
	if (handle)
	{
		long filesize = SetFilePointer(handle,0,0,FILE_END);
		SetFilePointer(handle,0,0,FILE_BEGIN);

		DWORD bytesread = 0,totalbytes = 0;
		PBYTE buffer = (PBYTE)winMalloc(chunkSize);

		// Prime it 1st
		long waittime = GetTickCount() + (secs * 1000);
		while((long)GetTickCount() < waittime)
		{
			if (ReadFile(handle,buffer,chunkSize,&bytesread,NULL)==FALSE)
			{
				winFree(buffer);
				return -1;
			}
		}

		waittime = GetTickCount() + 1000;
		while((long)GetTickCount() < waittime)
		{
			if (ReadFile(handle,buffer,chunkSize,&bytesread,NULL)==FALSE)
			{
				winFree(buffer);
				return -1;
			}
			totalbytes += bytesread;
		}

		winFree(buffer);
		SetFilePointer(handle,0,0,FILE_BEGIN);
		return (int)(totalbytes);
	}
	return -2;
}

void FRSRC::Close(void)
{
	if (handle)
	{
		CloseHandle(handle);
		handle=NULL;
	}

	winFree(info);
	info = NULL;
	winFree(directory);
	directory = NULL;

	maxFiles = 0;
	count = 0;
}

BOOL FRSRC::Open(const char *filename, int type, int max)
{
	FRSRC_FILEHEADER resourceheader;
	Close();

	// Open Resource File if we can
	switch(type)
	{
	case FRSRC_SEQ:
		handle = Open2Read(filename,FILE_FLAG_CDROM_READ);
		break;
	case FRSRC_RND:
		handle = Open2Read(filename,0);
		break;
	default:
		return FALSE;
	}

	if (handle==INVALID_HANDLE_VALUE) 
	{	// Error opening file
		return FALSE;
	}

	if (max < 1)
		max = 1;

	maxFiles = max;
	info = (FRSRC_INFO *)winCalloc(maxFiles,sizeof(FRSRC_INFO));

	if (info)
	{
		memset(info,0,sizeof(FRSRC_INFO) * maxFiles);
		// Read resource header
		if (ReadDatatype(handle,&resourceheader)==TRUE)
		{
			if (resourceheader.id==0xf300)
			{	// File is a resource file
				count = resourceheader.count;
				if (count < maxFiles)
				{
					maxFiles=count;
					if (count==0)
					{
						Close();
						return FALSE;
					}
				}	

				// Allocate space for directory entries
				directory = (FRSRC_FILEENTRY *)
							winCalloc(count,sizeof(FRSRC_FILEENTRY));
				if (directory)
				{
					// Read directory entries
					if (ReadDatatype(handle,directory,(DWORD)count)==TRUE)
					{
						return TRUE;
					}
				}
			}
		}
	}
	Close();
	return FALSE;
}

BOOL FRSRC::CloseFile(long h)
{
	if (!IsFileHandleValid(h))
		return FALSE;

	info[h].fileopened = 0;
	info[h].eof = 0;
	return TRUE;
}

long FRSRC::OpenFile(const char *filename)
{
	long entryid = LocateFile(filename);
	if (entryid==-1) 
		return -1;

	return OpenFileEntry(entryid);
}

long FRSRC::OpenFile(long id)
{
	long entryid = LocateFile(id);
	if (entryid==-1)
   {
     DebugString("Cant find id %d", id);
		return -1;
   }

	return OpenFileEntry(entryid);
}

BOOL FRSRC::Eof(long h)
{ 
	if (!IsFileHandleValid(h))
		return FALSE;

	FRSRC_INFO		*f = &info[h];
	if (f->fileopened)
	{
		return f->eof;
	}
	return TRUE; // File closed
}

long FRSRC::Read(long h, LPVOID buffer, long size)
{
	if (!IsFileHandleValid(h))
		return -1;

	FRSRC_INFO		*f = &info[h];
	FRSRC_FILEENTRY	*d = f->entry;
	f->eof = 0;

	if (f->fileopened)
	{
		DWORD bytesread;
		if (f->position + size > d->size)
		{
			size = d->size - f->position;
			if (size == 0)
			{
				f->eof = 1;
				return 0;
			}
		}
		SetFilePointer(handle,f->position + d->location,0,FILE_BEGIN);
		if (ReadFile(handle,buffer,size,&bytesread,NULL)==TRUE)
		{
			f->position += bytesread;
			if (f->position >= d->size)
			{
				f->eof = 1;
			}
			return (long)bytesread;
		}
	}
	return -1;
}

long FRSRC::Seek(long h, long position, DWORD from)
{
	if (!IsFileHandleValid(h))
		return -1;

	FRSRC_INFO		*f = &info[h];
	FRSRC_FILEENTRY	*d = f->entry;

	if (!f->fileopened)
		return -1;

	long p;
	switch(from)
	{
	case FRSRC_CURRENT:
		p = f->position + position;
		break;

	case FRSRC_END:
		p = d->size - position;
		break;

	case FRSRC_BEGIN:
	default:
		p = position;
		break;
	}

	if (p < 0)
		p = 0;

	if (p >= d->size)
		p = d->size - 1;

	f->position = p;
	return p;
}