//
// ---------------------------------------
// Parental control password entry program
// ---------------------------------------
//	This subroutine is intended to be hosted by a DLL loaded and executed
//	by an InstallShield script for the Atari GameFilm project. It prompts
//	the user for a password, and then writes the password and other
//	default game options out to the registry using the specified key.
//
//	This code can also serve the basis of the "change password" routine
//	internal to the GameFilm engine.
//
//	Author: Jack Thornton
//	for CetaSoft / Level Five, Inc.
//	952 Postal Way, Suite 3
//	Vista, CA 92083
//	(619) 631-5855
//
//	Client: Atari Corporation
//	Project: GameFilm Engine
//	Date: 3/21/96
// --------------------------------------------------------------------------

#include <windows.h>
#include "resource.h"
#include "dsetup.h"

// ----------------------------------------------------------------- CONSTANTS
#define PWD_SIZE    51

// ----------------------------------------------------------------- VARIABLES
// save the module ID so that we can find our resources
HINSTANCE hModule;

// options settings
char password[PWD_SIZE];
BOOL bMinimizeSex;
BOOL bMinimizeViolence;

// --------------------------------------------------------------------- TYPES
struct OptionsData
{
  // presentation
  BOOL subtitles;
  BOOL voiceovers;
  BOOL zoom;
  BOOL skipahead;
  BOOL entirepath;
  BOOL menubar;

  // subjects to minimize
  char password[PWD_SIZE];
  BOOL minViolence;
  BOOL minSex;

  // sweat
  int difficultyLevel;

  int volume;
};

// ------------------------------------------------------ PRE-INITIALIZED DATA
char instructions1[] = {
    "Please enter a password for activation of parental control features:"
};

char instructions2[] = {
    "Please re-enter your password for verification:"
};

char noMatchTitle[] = {
    "Password mismatch"
};

char noMatchMsg[] = {
    "The passwords you entered did not match. Please try again!"
};

char cancelTitle[] = {
    "Parental Control Warning"
};

char lpstrReasonNull[] = {
    "did not choose to enter a password"
};

char lpstrReasonCancel[] = {
    "elected to cancel password entry"
};

char cancelMsg[] = {
    "Because you %s, there is currently no "
    "password protection for the parental control features. If you wish to, "
    "you may enable password protection later by running the game, selecting "
    "\"Options\", clicking on the \"Parental Control\" tab, and then clicking "
    "the \"Change Password...\" button."
};

char regErrTitle[] = {
    "Installation Error"
};

char regErrMsg[] = {
    "There was an error writing the program options to the Windows Registry."
};

// ---------------------------------------------------------------- PROTOTYPES
BOOL CALLBACK PasswordDlgProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK SettingsDlgProc(HWND, UINT, WPARAM, LPARAM);
void EncryptPassword(char *string);
void DecryptPassword(char *string);

// ---------------------------------------------------------------------- CODE
BOOL APIENTRY
DllMain(HANDLE hInstDLL, DWORD reason, LPVOID reserved)
{
    // make the module instance handle globally visible
    if (reason == DLL_PROCESS_ATTACH)
	hModule = hInstDLL;

    return TRUE;
}

//
// Exported DirectXSetup function
//
// This is a PASCAL (not WINAPI) function which can be called from an
// InstallShield script
//

LONG PASCAL
SetupDirectX(char source_dir[256])
{

//***	  debug on hard drive
//***	  lstrcpy(&source_dir[3], "GAMEFILM\\DIRECTX\0");
   // final for CDR
   lstrcpy(&source_dir[3], "DIRECTX\0");

   return DirectXSetup(NULL, source_dir, DSETUP_DIRECTX);

}

BOOL PASCAL
SetParentalControl(char *lpstrKeyName)
{
HKEY hKey;		    // handle to the options key in the registry
OptionsData options;	    // data structure to write to registry
int rv; 		    // return value from dialog box

  try {
    do {
	// run the password setting dialog box
	DialogBox(hModule, "DLG_PASSWORD", HWND_DESKTOP, (DLGPROC)
	    PasswordDlgProc);

	// and the parental control settings box
	rv = DialogBox(hModule, "DLG_SETTINGS", HWND_DESKTOP, (DLGPROC)
	    SettingsDlgProc);
    } while (rv != TRUE);

    // encrypt the password
    EncryptPassword(password);

    // create the options data block
    memset(&options, 0, sizeof(OptionsData));	// zero everything to start
    options.subtitles = TRUE;	    // then set explicit defaults
    options.voiceovers = TRUE;
    options.skipahead = TRUE;
    options.entirepath = TRUE;
    options.menubar = TRUE;
    options.minSex = bMinimizeSex;  // and the user's wishes regarding editing
    options.minViolence = bMinimizeViolence;
    memcpy(options.password, password, PWD_SIZE);   // copy password data

    // write the default options (plus password) out to the registry
    // note: by convention, a "null string" password turns password protection
    // off (the user may click on the parental control checkboxes and the
    // "Change Password" button without first entering a prior password).
    if (RegCreateKey(HKEY_CURRENT_USER, lpstrKeyName, &hKey) != ERROR_SUCCESS)
	throw hKey;
    if (RegSetValueEx(hKey, "OPTIONS", 0, REG_BINARY, (CONST BYTE *)&options,
	sizeof(OptionsData)) != ERROR_SUCCESS)
	    throw hKey;
    RegCloseKey(hKey);

    // okay!
    return TRUE;
  }
  catch (HKEY) {
    // could not open or write to registry so BITCH
    MessageBeep(MB_ICONHAND);
    MessageBox(HWND_DESKTOP, regErrMsg, regErrTitle, MB_ICONSTOP | MB_OK);
    return FALSE;
  }
}

void
EncryptPassword(char *string)
{
    // stub
}

void
DecryptPassword(char *string)
{
    // stub
}

void
InitializePasswordEntry(HWND hDlg)
{
HWND hWndEntry;
HWND hWndRepeat;

    // clear the password
    password[0] = 0;

    // get the handles to our controls
    hWndEntry = GetDlgItem(hDlg, IDC_PASSWORD);
    hWndRepeat = GetDlgItem(hDlg, IDC_REPEATPW);

    // clear and enable the first entry box
    SendMessage(hWndEntry, WM_SETTEXT, 0, (LPARAM)password);
    EnableWindow(hWndEntry, TRUE);

    // clear and disable the second entry box
    SendMessage(hWndRepeat, WM_SETTEXT, 0, (LPARAM)password);
    EnableWindow(hWndRepeat, FALSE);

    // set the focus on the first box
    SetFocus(hWndEntry);

    // and tell the user what is expected of him/her
    SendDlgItemMessage(hDlg, ID_CAPTION, WM_SETTEXT, 0, (LPARAM) instructions1);
}

void
NoPasswordMessage(HWND hDlg, char *reason)
{
char compositeMsg[384];

    // create the composite message
    wsprintf(compositeMsg, cancelMsg, reason);

    // and present message to user
    MessageBeep(MB_ICONEXCLAMATION);
    MessageBox(hDlg, compositeMsg, cancelTitle, MB_OK | MB_ICONEXCLAMATION);
}

BOOL CALLBACK
PasswordDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
char pw1[PWD_SIZE];	// password string 1
char pw2[PWD_SIZE];	// password string 2
HWND hEditPwd;		// handle of edit control window

    switch (message) {
	case WM_INITDIALOG:
	    // initialize new password to NULL string
	    InitializePasswordEntry(hDlg);

	    // inform dialog box management code that we handled this message
	    return TRUE;

	case WM_COMMAND:
	    // what control is sending us this?
	    switch(LOWORD(wParam)) {
		case IDOK:
		    // look at currently focused control - is it edit 1?
		    hEditPwd = GetDlgItem(hDlg, IDC_PASSWORD);
		    if (GetFocus() == hEditPwd) {
			// yep - disable edit one
			EnableWindow(hEditPwd, FALSE);

			// and enable the second control (plus ACCEPT button)
			EnableWindow(GetDlgItem(hDlg, IDC_REPEATPW), TRUE);

			// change the instructions to the user
			SendDlgItemMessage(hDlg, ID_CAPTION, WM_SETTEXT, 0,
			    (LPARAM) instructions2);

			// move on to next control
			PostMessage(hDlg, WM_NEXTDLGCTL, 0, 0);
		    } else {
			// do both passwords match?
			SendDlgItemMessage(hDlg, IDC_PASSWORD, WM_GETTEXT, 50,
			    (LPARAM)(LPSTR)pw1);
			SendDlgItemMessage(hDlg, IDC_REPEATPW, WM_GETTEXT, 50,
			    (LPARAM)(LPSTR)pw2);
			if (lstrcmp(pw1, pw2) != 0) {
			    // Whoops! Inform user of mismatch
			    MessageBeep(MB_ICONEXCLAMATION);
			    MessageBox(hDlg, noMatchMsg, noMatchTitle,
				MB_OK | MB_ICONEXCLAMATION);

			    // re-initialize for retry
			    InitializePasswordEntry(hDlg);
			} else {
			    // yes, they match: copy password
			    lstrcpy(password, pw1);

			    // if password is "null" (no password), warn user
			    if (!password[0])
				NoPasswordMessage(hDlg, lpstrReasonNull);

			    // skip out of the dialog
			    EndDialog(hDlg, TRUE);
			}
		    }
		    break;

		case IDCANCEL:
		    // warn user about lack of password and go away
		    NoPasswordMessage(hDlg, lpstrReasonCancel);
		    EndDialog(hDlg, FALSE);
		    break;
	    }
	    break;
    }

    // tell dialog management code that we did NOT handle the message
    return FALSE;
}

BOOL CALLBACK
SettingsDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message) {
	case WM_COMMAND:
	    // what control is sending us this?
	    switch(LOWORD(wParam)) {
		case IDC_NEXT:
		    // set the variables depending on button state
		    bMinimizeSex = (BOOL) SendDlgItemMessage(hDlg, IDC_MINSEX,
			BM_GETCHECK, 0, 0);
		    bMinimizeViolence = (BOOL) SendDlgItemMessage(hDlg,
			IDC_MINVIOLENCE, BM_GETCHECK, 0, 0);

		    // go forward
		    EndDialog(hDlg, TRUE);
		    break;

		case IDC_PREV:
		    // user wants to go back to password setting
		    EndDialog(hDlg, FALSE);
		    break;
	    }
	    break;
    }

    // tell dialog management code that we did NOT handle the message
    return FALSE;
}

