/********************************************************************\

	Renderer Class Module
	by Rudy Mathieu

\********************************************************************/

#include "render.h"
#include "Dbgtst.h"

BOOL Renderer::Start(void)
{
	return DrawDibStart(dibHandle,0);
}

BOOL Renderer::Stop(void)
{
    return DrawDibStop(dibHandle);
}

BOOL Renderer::Begin(HDC hDC)
{
	return DrawDibBegin(dibHandle, hDC, destWidth, destHeight, 
      	   &bitmapInfo, sourceWidth, sourceHeight, NULL);
}

BOOL Renderer::End(void)
{
	return DrawDibEnd(dibHandle);
}

BOOL Renderer::FormatDifferent(BITMAPINFOHEADER *info)
{
   if (memcmp(&bitmapInfo,info,sizeof(bitmapInfo)))
	   return TRUE; /* Yes. */
   else
	   return FALSE;
}

void Renderer::Setup(BITMAPINFOHEADER *info,LPRECT rc)
{
   if (!FormatDifferent(info))
		return; /* No need to bother */

   memcpy(&bitmapInfo,info,sizeof(bitmapInfo));
   Off();
   if (SetDisplayAttr(rc)==TRUE)
   {
      Begin(dc);
	  Start();
	  On();
	  status |= RENDERER_READY;
	  return;
   }
   On();
}

void Renderer::Close(void)
{
   fader.Close();
   status &= ~RENDERER_READY;
   if (dibHandle)
   {
      Stop();
	  End();
	  DrawDibClose(dibHandle);
	  dibHandle = NULL;
   } 
   if (dc)
   {
      ReleaseDC(hwnd,dc);
      dc=NULL;
   }
   memset(&bitmapInfo, 0, sizeof(bitmapInfo));
}

BOOL Renderer::SetDisplayAttr(LPRECT rect)
{
    if (hwnd==NULL || dibHandle==NULL || dc==NULL)
	   return FALSE;

  if (rect==NULL || fullRect.right==0)
  {
    GetClientRect(hwnd,&fullRect);
    if (fullRect.bottom > 300)
    {
      fullRect.bottom=fullRect.top+384;
    }
    else
    {
      fullRect.bottom-=24;
    }
  }
  else
  {
    CopyRect(&fullRect,rect);
  }

    rect = &fullRect;
#if 0
  /* Correct for DWORD boundaries */
#define DWORD_MASK   (~0x03)
  destX = (rect->left + 3) & DWORD_MASK;
  destY = (rect->top + 3) & DWORD_MASK;
  destWidth  = (rect->right - destX + 1) & DWORD_MASK;
  destHeight = (rect->bottom - destY + 1) & DWORD_MASK;
#else
  destX = 0;
  destY = 0;
  if (fullRect.bottom > 300)
  {
    destWidth = 2*bitmapInfo.biWidth;
    destHeight = 2*bitmapInfo.biHeight;
  }
  else
  {
    destWidth = bitmapInfo.biWidth;
    destHeight = bitmapInfo.biHeight;
  }
#endif
	sourceWidth = bitmapInfo.biWidth;
	sourceHeight = bitmapInfo.biHeight;

	fader.Open(destWidth,destHeight);
	if (status & RENDERER_READY)
	{
		Stop();
		End();
		Begin(dc);
		Start();
	}

	return TRUE;
}
 
Renderer::Renderer() 
{ 
	soundfader=NULL;
	faderlevel = 0;
	fadeinc = 0;
	fadetype = NOFADE;

	status = 0; 
	dibHandle=NULL; 
	dc=NULL; 
	hwnd = NULL; 
	fullRect.left = fullRect.right = fullRect.top = fullRect.bottom = 0;
	memset(&bitmapInfo,0,sizeof(bitmapInfo));
	On();
}

BOOL __fastcall Renderer::Draw(LPVOID data,UINT flag)
{
  if (!(status & DISPLAY_ON) || !Ready())
    return FALSE;

//extern HWND thisWindow;
//DebugDataRate(thisWindow,1);

  if (fadetype==NOFADE)
  {
    return DrawDibDraw(dibHandle,dc, destX, destY, destWidth, destHeight,
      &bitmapInfo,data,0,0,sourceWidth,sourceHeight,flag
      |DDF_SAME_DRAW|DDF_SAME_HDC);
  }

  faderlevel += fadeinc;

  int newvolume = (FADEMAX-faderlevel) * -10000/FADEMAX;
  soundfader->SetVolume(newvolume);

  if (fadetype==FADEIN)
  {
    if (faderlevel > FADEMAX)
    {
      // Fader is fully IN so turn off
      fadetype=NOFADE;
      Begin(dc);
      Start();
      return Draw(data,flag);
    }
  } else {
    if (faderlevel < 0)
    {
      // Fader is fully OUT 
      faderlevel = 0;
      return PatBlt(dc,0,0,destWidth, destHeight,BLACKNESS);
    }
  }

  // Render into fader's DC
  if (destWidth == sourceWidth)
  {
    BOOL r = DrawDibDraw(dibHandle,fader.DC(), destX, destY, destWidth, destHeight,
      &bitmapInfo,data,0,0,sourceWidth,sourceHeight,flag );
    fader.UseFilter(faderlevel, 1, dc);
    return r;
  }
  else
  {
    BOOL r = DrawDibDraw(dibHandle,fader.DC(), destX, destY, destWidth, destHeight,
      &bitmapInfo,data,0,0,destWidth,destHeight,flag );

    fader.UseFilter(faderlevel, 2,dc); 
    return r;
  }
}

void __fastcall Renderer::Fade(FADETYPE t,int step, DSoundBuffer *snd)
{
	soundfader = snd;
	fadetype = t; 
	switch(t)
	{
	case NOFADE:
		Begin(dc);
		Start();
		if (soundfader)
			soundfader->SetVolume(0);
		return;

	case FADEIN:
		faderlevel = 0;
		fadeinc = abs(step);
		break;

	case FADEOUT:
		faderlevel = FADEMAX;
		fadeinc = -abs(step);
		break;
	}
	Begin(fader.DC());
	Start();
}
