//
// Utility to make splash screen resource DLL compressed bitmaps
//	Atari GameFilm Project
//	Author: Jack Thornton
//	for Level Five, Inc.
//	02-20-96
//

// -------------------------------------------------------- Includes
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <ctype.h>
#include "resource.h"
#include "lzwcomp.h"

// -------------------------------------------------------- Declarations
#define TRUE	    1
#define FALSE	    0
#define OKAY	    0

#define MSG_MAKEDATA	(WM_USER+1958)
#define MSG_REPORT_FILE (WM_USER+1964)

// -------------------------------------------------------- Types and structures
typedef struct bFile {
    struct bFile *next;
    char filename[264];
    int x;
    int y;
    int width;
    int height;
} BitmapFileInfo;

typedef struct {
    DWORD originalSize;
    DWORD compSize;
    WORD x;
    WORD y;
    WORD width;
    WORD height;
} CompFileHeader;

// -------------------------------------------------------- Prototypes
int PASCAL WinMain(HINSTANCE, HINSTANCE, LPSTR, int);
LRESULT CALLBACK MainAppWinProc(HWND, UINT, WPARAM, LPARAM);
void MakeData(void);
BOOL CALLBACK PacifierDialogProc(HWND, UINT, WPARAM, LPARAM);
BOOL DidUserAbort(void);
void OpenScript();
void CompressFiles();

// -------------------------------------------------------- Variables
char MainAppWinClassName[] = "MkSplashBits";

HWND hMainWindow;	    // handle to main frame window
HWND hPacifierDialog;	    // handle to pacifier dialog
HINSTANCE hInstance;	    // application instance
BOOL bUserAbort;	    // did the user press CANCEL during compile?
BitmapFileInfo *fileList;   // list of files to process
int numFiles;		    // number of files in the list
char pathname[264];	    // path of script and output files
char scriptFile[264];	    // script filename
char errorMessage[512];     // error message

// -------------------------------------------------------- Code
int PASCAL
WinMain(HINSTANCE hInst, HINSTANCE hPrevInstance, LPSTR lpszCmdLine,
	int nCmdShow)
{
WNDCLASS info;		// window class structure
MSG msg;		// current message being processed

    // fill out a structure for registering our window class
    info.style = CS_HREDRAW | CS_VREDRAW ;   // window redraws on resize
    info.lpfnWndProc = MainAppWinProc; // our message handler for the class
    info.cbClsExtra = 0;		// no extra class bytes
    info.cbWndExtra = 0;		// no extra window bytes
    info.hInstance = hInst;		// copied from global variable
    info.hIcon = NULL;
    info.hCursor = LoadCursor(NULL, IDC_ARROW);   // use standard arrow cursor
    info.hbrBackground = (HBRUSH) (COLOR_APPWORKSPACE + 1);
    info.lpszMenuName = NULL;
    info.lpszClassName = MainAppWinClassName;	// name of the class

    // register the class as defined by our info structure
    if (!RegisterClass(&info))
	return 0;

    // create our main window (an instance of our defined window class)
    hInstance = hInst;
    hMainWindow = CreateWindow(
	MainAppWinClassName,
	"Make splash screen data",
	WS_OVERLAPPED | WS_CAPTION | WS_THICKFRAME |
	  WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX |
	  WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE,
	CW_USEDEFAULT,
	CW_USEDEFAULT,
	CW_USEDEFAULT,
	CW_USEDEFAULT,
	HWND_DESKTOP,
	NULL,
	hInst,
	NULL
	);

    // process the messages that come to our program
    while (GetMessage(&msg, NULL, 0, 0)) {
	TranslateMessage(&msg);
	DispatchMessage(&msg);
    }

    // clean up and go
    UnregisterClass(MainAppWinClassName, hInstance);
    return msg.wParam;
}

// MainAppWinProc
// ---------------------------------------------------------------------
LRESULT CALLBACK
MainAppWinProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    // take the appropriate action for the type of message being sent
    switch (message) {
	case WM_CREATE:
	    // post message that starts the actual work
	    PostMessage(hWnd, MSG_MAKEDATA, 0, 0);
	    break;

	case MSG_MAKEDATA:
	    // create the data
	    MakeData();

	    // we are done - kill the window
	    DestroyWindow(hWnd);
	    return 0;

	case WM_DESTROY:
	    // we are outa here!
	    PostQuitMessage(0);
	    break;
    }

    // any message we don't handle (and some we do) gets passed to default proc
    return DefWindowProc(hWnd, message, wParam, lParam);
}

BOOL CALLBACK
PacifierDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message) {
	case WM_INITDIALOG:
	    // clear the user abort flag
	    bUserAbort = FALSE;
	    break;

	case WM_COMMAND:
	    // if user pressed cancel, set the flag for the abort proc to see
	    if (wParam == IDCANCEL) {
		bUserAbort = TRUE;
		return TRUE;
	    }
	    break;

	case MSG_REPORT_FILE:
	    SetDlgItemText(hDlg, IDC_REPORT, (LPSTR)lParam);
	    return TRUE;
    }

    return FALSE;
}

BOOL
DidUserAbort(void)
{
MSG msg;

    // handle messages
    while (!bUserAbort && PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
	if (!IsDialogMessage(hPacifierDialog, &msg)) {
	    TranslateMessage(&msg);
	    DispatchMessage(&msg);
	}
    }

    // and return current state of abort (or not)
    return bUserAbort;
}

void
OpenScript()
{
FILE *infd;		// handle to open script file
char line[256]; 	// script line buffer
char filename[256];
int x;
int y;
int w;
int h;
BitmapFileInfo *newEntry;
BitmapFileInfo *tail;
OPENFILENAME of;	// file-open dialog information structure
char *token;		// pointer to token
int lineNumber; 	// sequential line numbering

    // ask user for name (and path) of script file
    filename[0] = 0;
    of.lStructSize = sizeof(OPENFILENAME);
    of.hwndOwner = hMainWindow;
    of.hInstance = hInstance;
    of.lpstrFilter = NULL;
    of.lpstrCustomFilter = NULL;
    of.nMaxCustFilter = 0;
    of.nFilterIndex = 0;
    of.lpstrFile = scriptFile;
    of.nMaxFile = 262;
    of.lpstrFileTitle = NULL;
    of.nMaxFileTitle = 0;
    of.lpstrInitialDir = NULL;
    of.lpstrTitle = "Select script file";
    of.Flags = OFN_EXPLORER | OFN_LONGNAMES | OFN_NOVALIDATE;
    of.lpstrDefExt = NULL;
    if (!GetOpenFileName(&of))
	throw 1;		// user cancelled operation

    // copy path (sans filename) into "pathname"
    lstrcpy(pathname, scriptFile);	// first, copy entire thing
    token = strrchr(pathname, '\\');    // find the last backslash character
    if (token)
	*(++token) = 0; 		// and write a null terminator after it

    // open the file
    infd = fopen(scriptFile, "rt");
    if (!infd) {
	wsprintf(errorMessage, "Cannot open script file %s", scriptFile);
	throw errorMessage;
    }

    // read and parse the file, creating compression records
    lineNumber = 1;
    while (!feof(infd)) {
	// read a line
	if (fgets(line, 256, infd)) {
	    // does this line have any information?
	    if (line[0] && (line[0] != '\n') && (line[0] != ';')) {
		// parse line for information
		token = strtok(line, " ,");
		lstrcpy(filename, token);

		token = strtok(NULL, " ,");
		if (!token) {
notoken:	    wsprintf(errorMessage, "Badly formed script line %u",
			lineNumber);
		    throw errorMessage;
		}
		x = atoi(token);

		token = strtok(NULL, " ,");
		if (!token)
		    goto notoken;
		y = atoi(token);

		token = strtok(NULL, " ,");
		if (!token)
		    goto notoken;
		w = atoi(token);

		token = strtok(NULL, " ,");
		if (!token)
		    goto notoken;
		h = atoi(token);

		// create a new entry
		newEntry = (BitmapFileInfo *) malloc(sizeof(BitmapFileInfo));
		if (!newEntry) {
		    lstrcpy(errorMessage, "Not enough memory to read script");
		    throw errorMessage;
		}

		// fill in the entry
		lstrcpy(newEntry->filename, filename);
		newEntry->x = x;
		newEntry->y = y;
		newEntry->width = w;
		newEntry->height = h;
		newEntry->next = NULL;

		// add it to the list
		if (!fileList)
		    fileList = newEntry;
		else
		    tail->next = newEntry;
		tail = newEntry;

		// and increment the list size
		numFiles++;
	    }
	}
	lineNumber++;
    }

    // close the file
    fclose(infd);
}

void
MakeData()
{
    try {
	// open and parse the script file
	OpenScript();

	// open the "pacifier" modeless dialog (inform user of progress)
	hPacifierDialog = CreateDialog(hInstance, "DLG_PACIFIER", hMainWindow,
	    (FARPROC)PacifierDialogProc);

	// create the compressed files
	CompressFiles();

	// close the pacifier dialog
	DestroyWindow(hPacifierDialog);

	// tell the user the good news!
	MessageBox(hMainWindow, "Successful Completion!", "Results", MB_OK);
    }
    catch (char *errMsg) {
	DestroyWindow(hPacifierDialog);
	MessageBox(hMainWindow, errMsg, "Error", MB_OK | MB_ICONSTOP);
    }
    catch (int) {
	DestroyWindow(hPacifierDialog);
	MessageBox(hMainWindow, "User canceled operation", "Aborted",
	    MB_OK | MB_ICONSTOP);
    }
    catch (...) {
	DestroyWindow(hPacifierDialog);
	MessageBox(hMainWindow, "Unknown exception", "Error", MB_OK |
	    MB_ICONSTOP);
    }
}

void
MakeFilename(char *dest, char *src, char *ext)
{
char *extptr;

    // first, is there already a pathname inside the source name?
    if (strchr(src, '\\'))
	strcpy(dest, src);
    else
	wsprintf(dest, "%s%s", pathname, src);

    // do we need to replace the extension?
    if (ext) {
	// is there an extension to replace?
	if ((extptr = strchr(dest, '.')) != NULL)
	    lstrcpy(extptr, ext);
	else
	    lstrcat(dest, ext);
    }
}

void
CompressFiles()
{
BitmapFileInfo *sfile;	    // loop counter
HANDLE inFile;		    // input file handle
FILE *outFile;		    // output file handle
char fullSourceName[264];   // fully-qualified path name of file to open
char fullDestName[264];     // fully-qualified path name of file to create
char pMsg[256]; 	    // progress message
SECURITY_ATTRIBUTES security;	// file security options
DWORD sizeLO, sizeHIGH;     // size of input file
DWORD bytesRead;	    // number of bytes actually read
DWORD compressedSize;	    // size of compressed data
BYTE *dataBuffer;	    // raw bitmap data as read from BMP file
BYTE *compressionBuffer;    // compressed data buffer
CompFileHeader header;	    // the header for the output file

    // initialize the security attributes (generic)
    security.nLength = sizeof(SECURITY_ATTRIBUTES);
    security.lpSecurityDescriptor = 0;
    security.bInheritHandle = FALSE;

    // one at a time...
    for (sfile = fileList; sfile; sfile = sfile->next) {
	// has user aborted the program (be friendly and check)?
	if (DidUserAbort())
	    throw 1;

	// inform user of progress
	wsprintf(pMsg, "Compressing file %s", sfile->filename);
	SendMessage(hPacifierDialog, MSG_REPORT_FILE, 0, (LPARAM)pMsg);

	// create filenames
	MakeFilename(fullSourceName, sfile->filename, NULL);
	MakeFilename(fullDestName, sfile->filename, ".LZW");

	// open the specified input file
	inFile = CreateFile(fullSourceName, GENERIC_READ,
	    FILE_SHARE_READ, &security, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS,
	    NULL);
	if (inFile == INVALID_HANDLE_VALUE) {
	    wsprintf(errorMessage, "Couldn't open file %s, error code %u",
		sfile->filename, GetLastError());
	    throw errorMessage;
	}

	// find out how big the file is
	sizeLO = GetFileSize(inFile, &sizeHIGH);

	// allocate read buffer for that file
	dataBuffer = (BYTE *) malloc(sizeLO);
	if (!dataBuffer) {
	    CloseHandle(inFile);
	    wsprintf(errorMessage, "Unable to allocate %u bytes to load %s",
		sizeLO, sfile->filename);
	    throw errorMessage;
	}

	// also allocate a compression buffer
	compressionBuffer = (BYTE *) malloc(sizeLO);
	if (!compressionBuffer) {
	    CloseHandle(inFile);
	    free(dataBuffer);
	    wsprintf(errorMessage, "Unable to allocate %u bytes to load %s",
		sizeLO, sfile->filename);
	    throw errorMessage;
	}

	// load the file into the buffer
	ReadFile(inFile, dataBuffer, sizeLO, &bytesRead, NULL);
	if (bytesRead != sizeLO) {
	    CloseHandle(inFile);
	    free(dataBuffer);
	    wsprintf(errorMessage, "Error: read only %u of %u bytes of %s",
		bytesRead, sizeLO, sfile->filename);
	    throw errorMessage;
	}

	// close the input file
	CloseHandle(inFile);

	// compress the file
	compressedSize = CompressFile(compressionBuffer, dataBuffer, sizeLO);
	if (!compressedSize) {
	    if (curError == ERR_LARGER) {
		memcpy(compressionBuffer, dataBuffer, sizeLO);
		compressedSize = sizeLO;
	    } else {
		wsprintf(errorMessage, "COMPRESSOR ERROR: %s", getLastError());
		throw errorMessage;
	    }
	}

	// open the output file
	outFile = fopen(fullDestName, "wb");
	if (!outFile) {
	    wsprintf(errorMessage, "Unable to open output file %s", fullDestName);
	    throw errorMessage;
	}

	// write the output header (size plus rectangle)
	header.originalSize = sizeLO;
	header.compSize = compressedSize;
	header.x = sfile->x;
	header.y = sfile->y;
	header.width = sfile->width;
	header.height = sfile->height;
	if (fwrite(&header, 1, sizeof(header), outFile) != sizeof(header)) {
	    wsprintf(errorMessage, "Error writing header to %s", fullDestName);
	    throw errorMessage;
	}

	// write the compressed information
	if (fwrite(compressionBuffer, 1, compressedSize, outFile) != compressedSize) {
	    wsprintf(errorMessage, "Error writing data to %s", fullDestName);
	    throw errorMessage;
	}

	// close the output file
	fclose(outFile);

	// free up the memory
	free(dataBuffer);
	free(compressionBuffer);
    }
}

