/********************************************************************\
	
	Smart Handle
	by Rudy Mathieu

\********************************************************************/

#include <stdio.h>
#include "SHandle.h"
#include "Dbgtst.h"

#define SYNC(h,t)	(WaitForSingleObject((h),(t))!=WAIT_TIMEOUT)
#define UNSYNC(h)	ReleaseMutex((h))

long gChunkSize = (16 << 10);

int __fastcall SmartGroup::GetFreeBuffer(BOOL big)
{
	if (big==TRUE)
	{
		// Check loop buffer (Big buffer)
		if (!databuffer[LOOP_BUFFER].StatusFileOpened() &&
			!databuffer[LOOP_BUFFER].StatusLoaded())
		{
			return 0; // Focus is LOOP
		}
	} else {

		// Check standard buffer (small buffer)
		for (int i=OTHER_BUFFERS; i<numOfBuffers; i++)
		{
			static int index = 0;
			index++;
			if (index >= numOfBuffers)
				 index=OTHER_BUFFERS;

			if (!databuffer[index].StatusFileOpened() &&
				 databuffer[index].StatusEmpty() )
					return index;
		}
	}
	return -1;
}

BOOL SmartGroup::Setup(int buffers, int smallsize, int bigsize)
{
	if (buffers < 2)
		return FALSE;

	numOfBuffers = buffers;
    readerBusy = 0;

	databuffer = new SmartFile[buffers];
	if (databuffer==NULL)
	{
		Shutdown();
		return FALSE;
	}

	for (int i=0; i<buffers; i++)
	{
		smallBuffer = smallsize;
		bigBuffer = bigsize;

		int sz = smallBuffer;
		if (i==LOOP_BUFFER)
			sz = bigBuffer;

		if (databuffer[i].OpenBuffer(sz)==FALSE)
		{
			Shutdown();
			return FALSE;
		}
	}

	return TRUE;
}

void SmartGroup::Shutdown(void)
{
	if (databuffer)
	{
		delete[] databuffer;
		databuffer=NULL;
	}
}

int __fastcall SmartGroup::OpenEntry(long entryid, BOOL loop)
{
	int b;

	b=GetFreeBuffer(loop);		// Get free buffer
	if (b==-1)					
		return SH_BUFFER_NA;	// No free buffer

	if (readerBusy)
		return SH_BUSY;			// Reader Busy

	databuffer[b].ResetBuffer();// Ensure buffer is Reset
	if (databuffer[b].OpenFileEntry(entryid)==FALSE)
   {
     OutputDebugString("kina open it");
		return SH_ERROR_OPENING;// Error opening file
   }

	return b;
}

int __fastcall SmartGroup::Open(long id, BOOL loop)
{
	int b;

	b=GetFreeBuffer(loop);		// Get free buffer
	if (b==-1)					
		return SH_BUFFER_NA;	// No free buffer

	if (readerBusy)
		return SH_BUSY;			// Reader Busy

	databuffer[b].ResetBuffer();// Ensure buffer is Reset
	if (databuffer[b].OpenFile(id)==FALSE)
   {
     OutputDebugString("kina open it");
		return SH_ERROR_OPENING;// Error opening file
   }

	return b;
}

int __fastcall SmartGroup::Open(const char *filename, BOOL loop)
{
	int b;

	b=GetFreeBuffer(loop);		// Get free buffer
	if (b==-1)
		return SH_BUFFER_NA;	// No free buffer

	if (readerBusy)
		return SH_BUSY;			// Reader Busy

	databuffer[b].ResetBuffer();// Ensure buffer is Reset
	if (databuffer[b].OpenFile(filename)==FALSE)
   {
     OutputDebugString("kina open it");
		return SH_ERROR_OPENING;// Error opening file
   }

	return b;
}

void __fastcall SmartGroup::SetFileReadPeriod(int per)
{
	for (int i=0; i<numOfBuffers; i++)
		databuffer[i].SetFileReadPeriod(per);
	
}

/********************************************************************\
								Smart File
\********************************************************************/

// Returns TRUE when data is read or available
BOOL SmartFile::Check(long amount)
{
   if (fileHandle==-1 || buffer==NULL)
   {
	   return FALSE;
   }

   int s,e;
   s = End();

   if (amount==0)
      e = s + gChunkSize;
   else
	  e = s + amount;

   if (e > Size()) 
   {
	    StatusWrapped() = TRUE;
        s = 0; 
		e = gChunkSize;
   }
   if (DataInside(s,e)==FALSE)
   {
	   DWORD bytes;
	   BOOL result = StreamDataFromFile(s,&bytes);
       if ((result==TRUE && bytes==0) || result==FALSE)
	   {
		   // Finish reading. So let's close it up
		   StatusLoaded() = TRUE;
		   CloseFile();
		   return 0;
	   }
       End() = s + bytes;
   }
   return TRUE;
}


void __fastcall SmartFile::SetFileReadPeriod(int per)
{
	checkTimer.Set(per,SmartFileCheck,(LPVOID)this);
}

// Read data from file
BOOL __fastcall SmartFile::StreamDataFromFile(long start, PDWORD bytesRead)
{
	if (bigfile==NULL)
		return FALSE;

	*bytesRead = bigfile->Read(fileHandle,buffer + start,gChunkSize);
	if (*bytesRead==-1)
		return FALSE;

	return TRUE;
}

SmartFile::SmartFile()
{
    // We need a name unique to this object
	// Convert this pointer to text
	sprintf(SyncName,"SFSync%X\0",(long)this);

	terminateThread = FALSE;
	syncHandle = CreateMutex(NULL,FALSE,SyncName);

	SECURITY_ATTRIBUTES attr = { sizeof(attr),0,TRUE };
	threadHandle = CreateThread(&attr
		,0
		,(LPTHREAD_START_ROUTINE)AsynchFileHandler
		,(LPVOID)this
		,0
		,&threadId);
}

SmartFile::~SmartFile()
{
	terminateThread = TRUE;
	DWORD exitCode=1;
	for (;;)
	{
		BOOL result = GetExitCodeThread(threadHandle,&exitCode);
		if (exitCode==0 || result==FALSE)
			break;

		Sleep(0);
	}
	CloseHandle(syncHandle);
}

void __fastcall SmartFile::SpotCheck(int amount)
{
	if (SYNC(syncHandle,INFINITE))
	{
		Check(amount);
		UNSYNC(syncHandle);
	}
}

static void SmartFileCheck(LPVOID param)
{
	SmartFile *sf = (SmartFile *)param;

	if (SYNC(sf->threadSyncHandle,1000))
	{
		sf->Check();
		UNSYNC(sf->threadSyncHandle);
	}
}

static void AsynchFileHandler(LPVOID param)
{
	SmartFile *file = (SmartFile *)param;
	HANDLE syncHandle = OpenMutex(NULL,MUTEX_ALL_ACCESS,file->SyncName);
	file->threadSyncHandle = syncHandle;

	for (;;)
	{
		if (file->terminateThread==TRUE)
		{
			break;
		}

		file->checkTimer.TimeElapsed();
		Sleep(0);
	}
	CloseHandle(syncHandle);
	ExitThread(0);
}
