// ------------------------------------------------------------------------ //
//		       Windows Animation Engine (WANE)			    //
//									    //
//			    Author: Jack Thornton			    //
//		   Written for Studio Arts Multimedia, Inc.		    //
//			     All Rights Reserved			    //
// ------------------------------------------------------------------------ //
// FILE:	TIMING.CPP						    //
// DATE:	11/20/95						    //
// SYNOPSIS:	Timing mechanism for regular video/automation updates	    //
// NOTES:								    //
// ------------------------------------------------------------------------ //

#include "..\common\precomp.h"
#pragma hdrstop
#include "wane.h"

_timingMechanism::_timingMechanism()
{
}

_timingMechanism::~_timingMechanism()
{
    // discard history buffer
    if (history)
	free(history);
}

void
_timingMechanism::initialize(unsigned frameRate)
{
    // calculate ms interval by dividing 1000 by frame rate
    TimeFrameIncrement = 1000 / frameRate;

    // get current time
    TimeLastUpdate = timeGetTime();

    // create a history buffer (for calculating averaged frame rate)
    history = (unsigned *) malloc(frameRate * sizeof(unsigned));
    historyIndex = 0;
    historySize = frameRate;
}

void
_timingMechanism::registerFrameStart(void)
{
unsigned currentMark;

    // get current time mark
    currentMark = timeGetTime();

    // calculate current delta
    delta = currentMark - history[historyIndex];

    // and stash away the mark
    history[historyIndex++] = currentMark;

    // wrap index if necessary
    if (historyIndex >= historySize)
	historyIndex = 0;
}

int
_timingMechanism::reportTargetRate(void)
{
    return historySize;
}

int
_timingMechanism::reportActualRate(void)
{
    return (delta) ? (historySize * 1000) / delta : 0;
}

BOOL
_timingMechanism::hasTimerExpired(void)
{
    return (timeGetTime() - TimeLastUpdate) >= TimeFrameIncrement;
}

int
_timingMechanism::resetTimer(void)
{
unsigned numPeriods;	// number of whole frame periods that have elapsed

    numPeriods = (timeGetTime() - TimeLastUpdate) / TimeFrameIncrement;
    if (numPeriods)
	TimeLastUpdate += numPeriods * TimeFrameIncrement;
    return numPeriods;
}

void
_timingMechanism::close(void)
{
}

