/********************************************************************\

	SoundBite 		- Preloads entire sound
	SoundStream		- Stream from file

		Open(filename);
		Close();
		Reset();
		Play(loopcount);
		Stop();
		DonePlaying();

  lifecycle:
    Constructor
      Open-----------------------------]
        Play    ---] N times per open  ] N times per Construct
        Reset/Stop-]                   ]
      Close----------------------------]
    Destructor

\********************************************************************/

#ifndef _CWAVE 
#define _CWAVE 

#include "dsnd.h"
//#include "sptypes.h"

class FRSRC;

#define DONE_PLAYING	0x0001
#define SAMPLE_READY	0x0004
#define EOF_REACHED		0x0100

enum CW_Cycle { CW_READY, CW_OPEN, CW_PLAYING };
// constructor --> ready
// ready:   Open --> open
// open:    Play --> playing
// open:    Close--> ready
// playing: Stop --> open
// playing: Reset--> open

class CWaveReader;

class CWave
{
public:
   virtual ~CWave();
   CWave();
   void SetReader(CWaveReader*);
   BOOL Open(const char *filename);
   BOOL Open(int index); // not supported by all readers.
   void Close(void);
   void SetVolume(int vol)
   {
     if (dsound) dsound->SetVolume(vol);
   }

   virtual void Reset(void);
   virtual BOOL Play(int loopcount);
   BOOL Stop(void);
   BOOL DonePlaying(void) { return status.donePlaying; }
   BOOL ReallyDonePlaying()
   {
     return state == CW_READY || (DonePlaying() && dsound->DonePlaying());
   }

   friend void CALLBACK periodicWaveChecker(UINT,UINT,DWORD,DWORD,DWORD);
protected:
  BOOL StopUnguarded();

  CW_Cycle  state;     // can the operation be done? what's the state?
  LPVOID			sample;		// Sample Buffer
  struct StatusType
  {
    int sampleReady:1;
    int donePlaying:1;
    int eofReached:1;
  };
  StatusType status;

  DSoundBuffer  	*dsound;	// Direct Sound Handler
  DWORD 			soundSize;	// Sound Size in Bytes
  DWORD 			dataChunk;	// Size to read data by
  int 			loop;  		// Current Loop Countdown

  MMRESULT		timeEvent;	// TimeEvent
		
  void 	CloseSound(void);
  LONG 	ReadSoundBite(void);

  virtual DWORD 	SetupSound(DWORD size,WAVEFORMATEX *fmt);
  virtual BOOL 	Looping(void);
  virtual void 	Check(void);
  CWaveReader* reader;
private:
  LONG bytesToWrite;
  DWORD end_pos;
};

class SoundBite : public CWave
{
private:
   DWORD	SetupSound(DWORD size,WAVEFORMATEX *fmt);
   BOOL 	Looping(void);
	void 	Check(void);
public:
	void Reset(void);
	BOOL Play(int loopcount);
};

class SoundStream : public CWave { };

class CWaveReader
{
public:
  virtual ~CWaveReader() {}
  virtual BOOL Open(const char* filename) = 0;
  virtual BOOL Open(int index) { return FALSE; }
  virtual BOOL Read(BYTE* dest, LONG bytesToRead, LONG& bytesRead) = 0;
  virtual BOOL Rewind() = 0;
  virtual BOOL Close() = 0;
  virtual LONG GetLength() = 0;
  virtual BOOL GetFormat(WAVEFORMATEX& fmt) = 0;
};
class MMIOReader : public CWaveReader
{
public:
  MMIOReader() : h(0) {}
  BOOL Open(const char* filename);
  BOOL Read(BYTE* dest, LONG bytesToRead, LONG& bytesRead);
  BOOL Rewind();
  BOOL Close();
  BOOL GetFormat(WAVEFORMATEX& fmt);
  LONG GetLength();
private:
  HMMIO h;
  LONG topOfData;
  int length;
};
class ResourceReader : public CWaveReader
{
public:
  ResourceReader() : data(0),dataLength(0),position(0) {}
  BOOL Open(const char* filename);
  BOOL Read(BYTE* dest, LONG bytesToRead, LONG& bytesRead);
  BOOL Rewind();
  BOOL Close();
  LONG GetLength() { return dataLength; }
  BOOL GetFormat(WAVEFORMATEX& fmt);
private:
  int dataLength;
  WAVEFORMATEX fmt;
  BYTE* data;
  LONG position;
};
class BigFileWaveReader : public CWaveReader
{
public:
  BigFileWaveReader()
    : dataLength(0)
    , position(0)
    , dataStart(0)
    , handle(0)
  {
  }
  BOOL Open(const char* filename);
  BOOL Open(int index);
  BOOL Read(BYTE* dest, LONG bytesToRead, LONG& bytesRead);
  BOOL Rewind();
  BOOL Close();
  LONG GetLength() { return dataLength; }
  BOOL GetFormat(WAVEFORMATEX& fmt);
  static void SetArchive(FRSRC* bigfile_)
  {
    bigfile = bigfile_;
  }
private:
  static FRSRC* bigfile;
	long dataStart;
  int dataLength;
  long handle;
  WAVEFORMATEX fmt;
  LONG position;
};
#endif
