#include "fwl5.h"
#include "fileclip.h"
#include "film.h"
#include "navcontr.h"
#include "prompts.h"
#include "schdlr.h"
#include "options.h"
#include <stdexcpt.h>
#include <fstream.h>
#include "dbgtst.h"
#include "voxover.h"

FileManager* FileManager::it = 0; // access from statics.
extern GameOptions globalOptions;

//-----------------------------------------------------------------------------
struct LoopArrangement
{
  Scheduled loop;
  Scheduled leadins[3];
  BOOL startSkip[3];
};
static LoopArrangement loopArrangement;

//-----------------------------------------------------------------------------
#include "clcontex.h"
static ClipContext context;
//-----------------------------------------------------------------------------
static int clipPlayed = 0;
static int reminderPlayed = 0;
static int gate=0;
static int mouseCount = -1;

enum FMode 
{ FM_STARTING  // once per launch of process (unless it restores at the launch)
, FM_LOOPING   // just added a loop and its branches
, FM_HYPER     // just added a hyper node.
, FM_FAILHYPER // too few clips.
, FM_FAILLOOP  // the user failed to decide, so we decide
, FM_ENDADDED  // The game will end when queue empties.
, FM_RESTORED  // a game was just restored.
, FM_IDLE      // waiting to be resumed.
, FM_PAUSED    // waiting to be resumed from a pause.
, FM_CHANGECD  // waiting to be informed of cd change.
, FM_CHANGECDR // ... after which reminder will ensue.
};
static FMode fmode= FM_STARTING;
static FMode pausemode = FM_IDLE;
static int endKilled = 0;
//-----------------------------------------------------------------------------
// This trigger function is called at the last frame of the clip.
// 
void FileManager::KillScheduled(Film* f, void* what)
{
  CPNode* p = (CPNode*)what;
  it->OnEndClip(f, p);
}
//-----------------------------------------------------------------------------
// This trigger function is called at the last frame of the clip.
// 
void FileManager::KillReminder(Film* f, void* what)
{
  reminderPlayed=1; // from each reminder clip.
  CPNode* p = (CPNode*)what;
  it->OnEndClip(f, p);
  if (p->data.film.type == CP_LOOP)
  {
    it->scheduler->SelectLeadin(FIRST);
  }
}
//-----------------------------------------------------------------------------
void FileManager::KillLeadin(Film* f, void* what)
{
  CPNode* p = (CPNode*)what;
  it->promptManager->SetCurrentClip(0);
  it->voiceOverManager->SetClip(0);
}
static BOOL deadLoop = FALSE; // set when loop's branch is determined
//-----------------------------------------------------------------------------
void FileManager::KillLoop(Film* f, void* what)
{
  CPNode* p = (CPNode*)what;
  context.ClipEnd(p, f);
  // assume difflvls easy=0,medium=1,hard=2
  // require loops easy=4,medium=3,hard=2
  // formula 4-difflvl
  unsigned maxLoops = 4-globalOptions.GetDifficultyLevel();
  if ((f->GetLoopCount()==maxLoops-1 && mouseCount==-1))
  {
    if (!deadLoop && fmode == FM_LOOPING)
    {
      fmode = FM_FAILLOOP;
      gate = 0;
      deadLoop = TRUE;
    }
  }
  else if (mouseCount!=-1)
  {
    it->promptManager->SetCurrentClip(0);
    it->voiceOverManager->SetClip(0);
  }
}
//-----------------------------------------------------------------------------
// This trigger function is called upon the first frame of the clip.
// It arranges for another trigger to occur upon the last frame.
void FileManager::ResetReminder(Film* f, void* what)
{
  CPNode* p = (CPNode*)what;
  it->OnStartClip(f, p);
  mouseCount=-1;
  clipPlayed = TRUE;
//  f->AddCallback(f->Header()->totalframes-1, KillReminder, p);
  f->AddCallback(f->GetTotalFrames()-1, KillReminder, p);
}
//-----------------------------------------------------------------------------
// This trigger function is called upon the first frame of the clip.
// It arranges for another trigger to occur upon the last frame.
void FileManager::ResetScheduled(Film* f, void* what)
{
  it->reminding = FALSE;
  clipPlayed = TRUE;
  deadLoop = FALSE;
  CPNode* p = (CPNode*)what;
  it->OnStartClip(f, p);
  mouseCount=-1;
  f->AddCallback(f->GetTotalFrames()-1, KillScheduled, p);
}
//-----------------------------------------------------------------------------
static int leadinSelected = FALSE; // debugging.
void FileManager::ResetLoop(Film* f, void* what)
{
  it->reminding = FALSE;
  clipPlayed = TRUE;
  CPNode* p = (CPNode*)what;
  if (f->GetLoopCount()==0)
  {
    leadinSelected = FALSE;
    it->OnStartClip(f, p);
    f->AddCallback(f->GetTotalFrames()-1, KillLoop, p);
  }
  else
  {
    context.ClipStart(p, f);
  }
}
//-----------------------------------------------------------------------------
void FileManager::ResetLeadin(Film* f, void* what)
{
  it->reminding = FALSE;
  clipPlayed = TRUE;
  deadLoop = FALSE;
  CPNode* p = (CPNode*)what;
  it->OnStartClip(f, p);
  mouseCount=-1;
  gate = 0;
  f->AddCallback(f->GetTotalFrames()-1, KillLeadin, p);
}
//-----------------------------------------------------------------------------
CEvent* FileManager::AddOneFile(CPNode& node, FADETYPE fade)
{
  //DEBUGSTRING(("Adding %s\n", net.GetClipName(&node)));
  FILMTYPE ft = SHOT;
  switch(node.data.film.type)
  {
  case CP_LOOP: ft=LOOP; break;
  case CP_LEADIN: ft=LEADIN; break;
  case CP_LEADOUT: ft=LEADOUT; break;
  }
  CEvent* ev = scheduler->AddEntry((node.data.film.clip%65536), ft, fade);
  assert(ev != 0);
  CPNode* nd = const_cast<CPNode*>(&node);
  if (node.data.film.tracks)
  {
    int i; 
    int kTrack = 0;
    CPTrack* track = &node.data.film.pTracks[kTrack];
    int nSegments = track->segments;
    for (i=0; i<nSegments; i++)
    {
      ev->AddTrigger(track->pSegments[i].firstFrame, SSegmentOn, nd);
      ev->AddTrigger(track->pSegments[i].lastFrame, SSegmentOff, nd);
    }
  }
  if (node.data.film.rule == CP_HYPER)
  {
    ev->AddTrigger(node.data.film.data.startFrame, HyperOn, nd);
    ev->AddTrigger(node.data.film.data.endFrame, HyperOff, nd);
  }
  if (ft==LOOP)
  {
    ev->AddTrigger(0, ResetLoop, nd);
  }
  else if (ft==LEADIN)
  {
    ev->AddTrigger(0, ResetLeadin, nd);
  }
  else
  {
    ev->AddTrigger(0, ResetScheduled, nd);
  }
  return ev;
}
//-----------------------------------------------------------------------------
CEvent* FileManager::AddReminderFile(CPNode& node, FADETYPE how)
{
  CEvent* ev = scheduler->AddEntry(node.data.film.clip, SHOT, how);
  assert(ev != 0);
  CPNode* nd = const_cast<CPNode*>(&node);
  ev->AddTrigger(0, ResetReminder, nd);
  return ev;
}
//-----------------------------------------------------------------------------
BOOL FileManager::SaveGame(const char* n) const
{
  ofstream output(n, ios::out | ios::binary);
  if (output.good())
  {
    char buf[256];
    strcpy(buf, "GFCPMRJJ");
    output.write(buf, 8);
    strcpy(buf, controller->GetRegKey().c_str());
    output.write(buf, 256);
    extern GameOptions globalOptions;
    OptionsData options = globalOptions.GetData();
    output.write((char*)&options, sizeof(options));
    output.write((char*)psych.data, sizeof(psych.data));
    output.write((char*)&score, sizeof(score));
    playerState.SaveNodes(output, net.GetBaseAddress());
    net.SaveCounts(output);
  }
  return TRUE;
}
//-----------------------------------------------------------------------------
NCError FileManager::RestoreGame(const char* n)
{
  ifstream input(n, ios::in | ios::binary);
  if (input.good())
  {
    char buf[256];
    input.read(buf, 8);
    buf[8]=0;
    if (strcmp(buf,"GFCPMRJJ")==0)
    {
      input.read(buf, 256);
      string oldKey = controller->GetRegKey();
      NCError ec = controller->SetRegKey(buf);
      if (ec == NC_SUCCESS)
      {
        extern GameOptions globalOptions;
        OptionsData options;
        if (net.ReadFromArchive(bigfileHD, 0))
        {
          input.read((char*)&options, sizeof(options));
          input.read((char*)&psych.data, sizeof(psych.data));
          input.read((char*)&score, sizeof(score));
          playerState.LoadNodes(input, net.GetBaseAddress());
          net.ReadCounts(input);
          globalOptions.SetData(options);
          //
          // Here we should call things so that
          // Windowing option changes are effective.
          //
          loadIt = playerState.GetLastNode();
          if (loadIt == 0)
          {
            DEBUGSTRING(("getlastnode in restoregame made loadIt zero\n"));
          }
          scheduler->ForgetAll();
          fmode = FM_IDLE; //fmode = FM_RESTORED;
          currentDisk = 0; // might need to switch before playing
          return NC_SUCCESS;
        }
        return NC_NOFILE;
      }
      return ec;
    }
  }
  return NC_NOFILE;
}
//-----------------------------------------------------------------------------
void FileManager::DoReminder()
{
  reminderPointer.Reset();
  reminding = TRUE;
  loadIt = 0;
  DEBUGSTRING(("DoReminder made loadit zero.\n"));
  scheduler->ForgetAll();
  scheduler->Resume();
  
  //int newDisk;
  if (DifferentDiskReminder(newDisk))
  {
    fmode = FM_CHANGECDR;
  }
  else
  {
    AddReminderClips();
  }
}
//-----------------------------------------------------------------------------
void FileManager::DoRetry()
{
  Psych newPsych;
  long newScore;
  loadIt = playerState.GetNodeAndRetry(newPsych, newScore);
  if (loadIt==0)
  {
    DEBUGSTRING(("getnodeandretry made loadit zero in DoRetry\n"));
  }
  psych = newPsych;
  score = newScore;
  scheduler->ForgetAll();
  reminding = 0;
  AddClipsToPlay();
  scheduler->Resume();
}
//-----------------------------------------------------------------------------
CPNode* GetNextLeadOut(CPNode* start, PlayerStateIterator& i)
{
  PlayerStateIterator st = i;
  CPNode* node = start;
  while (1)
  {
    if (node->data.film.type == CP_LEADOUT)
    {
      return node;
    }
    if (node->data.film.type == CP_LOOP)
    {
      i = st;
      return 0;
    }
    if (i.Done())
    {
      i = st;
      return 0;
    }
    node = i.GetNode();
    i.Advance();
  }
  return 0;
}
//-----------------------------------------------------------------------------
void FileManager::AddReminderClips()
{
  int clips = 0;
  clipPlayed = 0;
#if 0
  if (!globalOptions.EntirepathOn())
  {
    reminderPointer.SetToEnd();
    //reminderPointer.Backup();
    int found=0;
    while (!reminderPointer.RDone())
    {
      loadIt = reminderPointer.GetNode();
      if (loadIt->data.film.type == CP_LOOP)
      {
        found =1;
        break;
      }
      reminderPointer.Backup();
    }
    if (!found)
    {
      reminderPointer.Reset();
    }
  }
  else
  {
    reminderPointer.Reset();
  }
#else
  reminderPointer = GetReminderStart();
  loadIt = reminderPointer.GetNode();
#endif
  while (!reminderPointer.Done())
  {
    loadIt = reminderPointer.GetNode();
    if (loadIt==0)
    {
      DEBUGSTRING(("reminderPointer.GetNode() made loadit zero in ManageReminder.\n"));
    }
    reminderPointer.Advance();
    if (globalOptions.SkipAheadOn()
       && loadIt->data.film.type == CP_LEADIN
       )
    {
      CPNode* leadOut = GetNextLeadOut(loadIt, reminderPointer);
      if (leadOut)
      {
        AddReminderFile(*loadIt, FADEOUT);
        clips++;
        AddReminderFile(*leadOut, FADEIN);
        clips++;
        loadIt = leadOut;
      }
      else
      {
        AddReminderFile(*loadIt, NOFADE);
        clips++;
      }
    }
    else
    {
      AddReminderFile(*loadIt, NOFADE);
      clips++;
    }
  }
  loadIt = net.GetCompatibleLink(loadIt->pLinks[0], psych);
  if (loadIt!=0)
  {
    AddClipsToPlay();
    assert(fmode != FM_IDLE); 
    // if it's idle, we're toast.
  }
  else
  {
    fmode = FM_ENDADDED;
  }
}
//-----------------------------------------------------------------------------
inline int DiskFromID(int id)
{
  int diskBits = id/65536;
  int diskNo =-1;
  while (diskBits) 
  {
    diskBits/=2;
    diskNo++;
  }
  return diskNo;
}
//-----------------------------------------------------------------------------
void FileManager::ResumePlayback()
{
  if (fmode == FM_PAUSED)
  {
    fmode = pausemode;
  }
  if (fmode == FM_IDLE
    || fmode == FM_RESTORED
    )
  {
    if (loadIt != 0)
    {
      if (DiskFromID(loadIt->data.film.clip) != currentDisk)
      {
        newDisk = DiskFromID(loadIt->data.film.clip);
        fmode == FM_CHANGECD;
      }
      else
      {
        AddClipsToPlay();
      }
    }
  }
  DEBUGSTRING(("Resume "));
  scheduler->DebugReport();
  scheduler->Resume();
}
//-----------------------------------------------------------------------------
void FileManager::PausePlayback()
{
  if (fmode != FM_PAUSED)
  {
    DEBUGSTRING(("Pause "));
    scheduler->DebugReport();
    scheduler->Pause();
    pausemode = fmode;
    fmode = FM_PAUSED;
  }
}
//-----------------------------------------------------------------------------
static int finishSkipSequence = 0;
static CPNode* finishSkipFrom = 0;
//-----------------------------------------------------------------------------
// AddClipsToPlay:
// Add clips to the scheduler queue until the end is reached,
// or a loop is reached.
// If the sequence ends on a loop, the leadins out of the loop are predicted
// and queued, complete with the fadeout effect if they will need it.
// We also leave bread crumbs so that we can set up the fade in and skip
// if the leadin that we actually used was a fade in.
void FileManager::AddClipsToPlay()
{
  clipPlayed = 0;
  assert(loadIt != 0);
  while (loadIt != 0)
  {
    if (loadIt->data.film.type == CP_LEADIN)
    {
      CPNode* leadOut = 0;
      if (loadIt->data.film.viewed 
         && globalOptions.SkipAheadOn()
         && (leadOut = net.GetNextLeadout(loadIt, psych))
         )
      {
        AddOneFile(*loadIt, FADEOUT);
        AddOneFile(*leadOut, FADEIN);
        loadIt = leadOut;
      }
      else
      {
        AddOneFile(*loadIt, NOFADE);
      }
    }
    else if (loadIt->data.film.type == CP_LOOP)
    {
      EnableAllBranches();
      loopArrangement.loop.node = loadIt;
      loopArrangement.loop.event = AddOneFile(*loadIt, NOFADE);
      // Grumble.
      // I decided to avoid problems elsewhere
      // by loading a leadin even for a disabled path.
      // The trick is to load a good leadin.
      // the kfilled variable gives me some (any) used
      // leadin in case one slot isn't used.
      // We use it for the otherwise unused branches.
      int i;
      int kfilled = -1;
      for (i=0; i<3; i++)
      {
        CPNode* link = net.GetCompatibleLink(loadIt->pLinks[i], psych);
        loopArrangement.leadins[i].node = link;
        if (link)
        {
          kfilled = i;
          assert(link->nodeType != CPN_CONTROL);
          // The leadin skip logic is cast this
          // way for ease of debugging.
          BOOL ss1 = link->data.film.viewed;
          BOOL ss2 = globalOptions.SkipAheadOn();
          CPNode* trialLeadout = net.GetNextLeadout(link, psych);
          BOOL ssf = ss1 && ss2 && (trialLeadout != 0);
          loopArrangement.startSkip[i] = ssf;
        }
        else
        {
          DisableBranch(i);
        }
      }
      // If branches are unused, assign node pointers to them.
      for (i=0; i<3; i++)
      {
        if (loopArrangement.leadins[i].node == 0)
        {
          loopArrangement.leadins[i] = loopArrangement.leadins[kfilled];
        }
      }
      // Tap the scheduler and queue up the clips for each
      // branch based on the node pointers assigned to
      // each branch.
      for (i=0; i<3; i++)
      {
        BOOL ssf = loopArrangement.startSkip[i];
        CPNode* link = loopArrangement.leadins[i].node;
        CEvent* newEvent = AddOneFile(*link, ssf?FADEOUT:NOFADE);
        loopArrangement.leadins[i].event = newEvent;
      }
      fmode = FM_LOOPING;
      // loadIt is still the loop node.
      break;
    }
    else if (loadIt->data.film.rule == CP_HYPER)
    {
      AddOneFile(*loadIt, NOFADE);
      fmode = FM_HYPER;
      // loadIt is still the hyper node.
      break;
    }
    else
    {
      if (finishSkipSequence)
      {
        CPNode* leadOut = net.GetNextLeadout(finishSkipFrom, psych);
        if (leadOut)
        {
          AddOneFile(*leadOut, FADEIN);
          loadIt = leadOut;
        }
        finishSkipSequence = 0;
      }
      else
      {
        AddOneFile(*loadIt, NOFADE);
      }
    } // end non-loop.
    if (loadIt->links == 0)
    {
      fmode = FM_ENDADDED;
      loadIt = 0;
      DEBUGSTRING(("loadit==0 and FM_ENDADDED.\n"));
      // Now we won't add more clips until
      // ResumeClip is called.
    }
    else
    {
      CPNode* pnode = loadIt;
      loadIt = net.GetCompatibleLink(loadIt->pLinks[0], psych, TRUE);
      if (loadIt == 0)
      {
        DEBUGSTRING(("GetCompatibleLinkloadit==0 and FM_ENDADDED.\n"));
        fmode = FM_ENDADDED;
      }
      else if (loadIt->nodeType == CPN_CONTROL
        && loadIt->data.control.controlType==CPC_CHANGEDISK
        )
      {
        newDisk = loadIt->data.control.data.newDisk;
        loadIt = loadIt->pLinks[0];
        fmode = FM_CHANGECD;
        return;
      }
    }
  } // end while
}
//-----------------------------------------------------------------------------
void FileManager::Manage()
{
  if (fmode==FM_STARTING)
  {
    if (!net.ReadFromArchive(bigfileHD, 0))//!net.ReadFromFile("caves_1d.moo"))
    {
      throw logic_error("Couldn't open moo file.");
    }
    loadIt = net.GetFirstNode();
    memset(&psych, 0, sizeof(psych));
    score = 0;
    AddClipsToPlay();
  }
  else if (fmode==FM_FAILHYPER)
  {
    loadIt = net.GetCompatibleLink(loadIt->pLinks[1], psych);
    if (loadIt==0)
    {
      fmode = FM_ENDADDED;
    }
    else
    {
      AddClipsToPlay();
    }
  }
  else if (fmode==FM_FAILLOOP)
  {
    assert(loadIt != 0);
    assert(!leadinSelected);
    leadinSelected = TRUE;
    scheduler->SelectLeadin(THIRD);
    if (loopArrangement.startSkip[2])
    {
      finishSkipSequence = TRUE;
      finishSkipFrom = loopArrangement.leadins[2].node;
      DEBUGSTRING(("Branched to a skipper on failed loop\n"));
    }
    loadIt = net.GetCompatibleLink(loopArrangement.loop.node->pLinks[2], psych, TRUE);
    assert(loadIt != 0);
    assert(loadIt->nodeType == CPN_FILM);
    assert(loadIt->data.film.type==CP_LEADIN);
    loadIt = loopArrangement.leadins[2].node;
    assert(loadIt != 0);
    assert(loadIt->nodeType == CPN_FILM);
    assert(loadIt->data.film.type==CP_LEADIN);
    loadIt = loadIt->pLinks[0]; // point to its shot.
    assert(loadIt != 0);
    assert(loadIt->nodeType == CPN_FILM);
    assert(loadIt->data.film.type!=CP_LEADIN);
    DEBUGSTRING(("FailLoop count s/b 0\n"));
    scheduler->DebugReport();
    AddClipsToPlay();
  }
  else if (fmode==FM_ENDADDED)
  {
    // if clips played but nothing plays now...
    if (clipPlayed && curNode == 0 && curClip==0)
    {
      fmode = FM_IDLE;
    }
  }
  else if (fmode==FM_CHANGECD)
  {
    if (clipPlayed 
      && scheduler->GetTotalEventsInCue()==0 
      && !scheduler->IsClipPlaying()
      )
    {
      controller->ChangeCD(1);
    }
  }
  if (fmode==FM_ENDADDED || fmode==FM_LOOPING)
  {
    if (reminderPlayed && !reminding)
    {
      reminderPlayed=0;
      controller->ReminderEnds();
    }
  }
  // Let's compute the current segment here.
  WatchSegments();
}
//-----------------------------------------------------------------------------
void FileManager::TakeABranch()
{
  if (context.hyperGate && fmode == FM_HYPER)
  {
    context.hyperClicks++;
    if (context.hyperClicks >= context.node->data.film.data.buttonHits)
    {
      loadIt = net.GetCompatibleLink(context.node->pLinks[0], psych, TRUE);
      AddClipsToPlay(); // resolve and end hyper segment.
    }
  }
  else if (context.segOn && context.node->data.film.type == CP_SHOT && context.shotGate)
  {
    CPSegment* segments = context.node->data.film.pTracks[context.trackIndex].pSegments;
    psych += segments[context.segIndex].psych;
    context.shotGate=FALSE;
  }
  else if (gate)
  {
    CPSegment* segments = context.node->data.film.pTracks[context.trackIndex].pSegments;
  //int kTrack = curKSeg;
    if (BranchEnabled(context.segIndex) && !deadLoop)
    {
      deadLoop = TRUE;
      mouseCount = context.segIndex;
      promptManager->SetMouseNode(context.node);
      gate = 0;
      assert(!leadinSelected);
      leadinSelected = TRUE;
      scheduler->SelectLeadin(context.segIndex?SECOND:FIRST);
      //
      Psych oldPsych = psych;
      psych += segments[context.segIndex].psych;
      score += segments[context.segIndex].completion;
      //
      assert(fmode == FM_LOOPING);

      // Grab the lead in to start walking nodes again.
      if (loopArrangement.startSkip[context.segIndex])
      {
        finishSkipSequence = TRUE;
        DEBUGSTRING(("Branched to a skipper\n"));
        loadIt = net.GetCompatibleLink(curNode->pLinks[context.segIndex], oldPsych, TRUE);
        loadIt = loadIt->pLinks[0];
        finishSkipFrom = loadIt;
      }
      else
      {
        loadIt = net.GetCompatibleLink(curNode->pLinks[context.segIndex], oldPsych, TRUE);
        loadIt = loadIt->pLinks[0];
      }
      DEBUGSTRING(("Choose Loop count s/b 1\n"));
      scheduler->DebugReport();
      AddClipsToPlay();
    }
  }
}

//-----------------------------------------------------------------------------
const Psych& FileManager::GetPsych() const
{
  return psych;
}

//-----------------------------------------------------------------------------
BOOL FileManager::IsPaused()
{
  return fmode == FM_PAUSED;
}

//-----------------------------------------------------------------------------
BOOL FileManager::IsOver()
{
  return (fmode == FM_IDLE || (fmode==FM_PAUSED && pausemode==FM_IDLE))
    && !scheduler->IsClipPlaying()
    ;
}

//--------------------------------------------------------------------
void FileManager::OnStartClip(Film* f, CPNode* n)
{
  curClip = f;
  curNode = n;
  context.ClipStart(n, f);
  if (!reminding)
  {
    playerState.VisitNode(n, psych, score);
    n->data.film.visits++;
    n->data.film.viewed = TRUE;
    DEBUGSTRING(("%s\n", net.GetClipName(n)));
  }
  promptManager->SetCurrentNode(n);
  promptManager->SetCurrentClip(f);
  voiceOverManager->SetNode(n);
  voiceOverManager->SetClip(f);
}

//--------------------------------------------------------------------
void FileManager::OnEndClip(Film*, CPNode*)
{
  promptManager->SetCurrentClip(0);
  voiceOverManager->SetClip(0);
  context.ClipEnd(0,0);
  curClip = 0;
  curNode = 0;
}

//-----------------------------------------------------------------------------
void FileManager:: HyperOn(Film* f, void* d)
{
  CPNode* node = (CPNode*)d;
  context.HyperSegStart(node, f);
}

//-----------------------------------------------------------------------------
void FileManager::HyperOff(Film* f, void* d)
{
  CPNode* node = (CPNode*)d;
  context.HyperSegEnd(node, f);
  if (fmode == FM_HYPER)
  {
    fmode = FM_FAILHYPER;
  }
}
//-----------------------------------------------------------------------------
void FileManager::SSegmentOn(Film* f, void* d)
{
  CPNode* node = (CPNode*)d;
  context.ClipSegStart(node, f);
}

//-----------------------------------------------------------------------------
void FileManager::SSegmentOff(Film* f, void* d)
{
  CPNode* node = (CPNode*)d;
  context.ClipSegEnd(node, f);
}

//--------------------------------------------------------------------
// WatchSegments:
// In order to avoid reentrancy issues, prompts are drawn and
// voice-overs called up from the 'idle' level instead of from the
// callback functions. The callback functions are currently called
// from within a mmsystem timer event.
//
void FileManager::WatchSegments()
{
  if (context.changed)
  {
    context.changed = FALSE;
    if (context.isHyperSeg)
    {
      if (context.segOn)
      {
        promptManager->StartHyper(context.node->data.film.data.prompt);
        context.hyperGate = 1;
      }
      else
      {
        promptManager->EndHyper();
        context.hyperGate = 0;
      }
    }
    else
    {
      CPSegment* seg = context.node->data.film.pTracks[context.trackIndex].pSegments+context.segIndex;
      BOOL looping = context.node->data.film.type == CP_LOOP;
      if (looping && context.segOn)
      {
         // if waiting for loop choice.
        if (fmode == FM_LOOPING)
        {
          if (context.passCount > 0)
          {
            gate = 1; // allow a choice now.
            mouseCount = -1; // no prior choice accepted.
          }
          else
          {
            gate = 0;
          }
        }
      }
      if (looping && !context.segOn)
      {
        if (fmode == FM_LOOPING)
        {
          gate = 0; // disallow choices now.
        }
      }
      if (context.segOn)
      {
        // Of course, don't show prompts any more if
        // the player has selected a branch.
        BOOL loopSegAllowed = !deadLoop;
        if (loopSegAllowed || !looping)
        {
          voiceOverManager->NotifyOn(context);
          promptManager->NotifyOn(context);
        }
      }
      else
      {
        voiceOverManager->NotifyOff();
        promptManager->NotifyOff();
      }
    }
  }
}

//----------------------------------------------------------------------------
void FileManager::StartOver()
{
  loadIt = net.GetFirstNode();
  newDisk = DiskFromID(loadIt->data.film.clip);
  net.ZeroCounts();
  memset(&psych, 0, sizeof(psych));
  reminding = 0;
  scheduler->ForgetAll();
  if (newDisk == currentDisk)
  {
    AddClipsToPlay();
  }
  else 
  {
    fmode = FM_CHANGECD;
  }
}

//----------------------------------------------------------------------------
void FileManager::EnableAllBranches()
{
  disabledBranches = 0;
  promptManager->EnableAllBranches();
  voiceOverManager->EnableAllBranches();
}

//----------------------------------------------------------------------------
void FileManager::DisableBranch(int kBranch)
{
  disabledBranches |= (1<<kBranch);
  promptManager->DisableBranch(kBranch);
  voiceOverManager->DisableBranch(kBranch);
}

//----------------------------------------------------------------------------
BOOL FileManager::BranchEnabled(int kb)
{
  return !(disabledBranches & (1<<kb));
}

//----------------------------------------------------------------------------
long FileManager::GetScore() const
{
  return score;
}
const char* names[] = 
{
  "Aggressive", "Decisive", "Analytical", "Sexual", "Energetic"
};
const char* tmplt = "You were %s yet %s, but not too %s.";
//----------------------------------------------------------------------------
// GetAnalysis:
// key-sort the numbers, then use those indices
// against the array of names. sprintf the names
// into a sentence. It's kinda like Astrology.
//
string FileManager::GetAnalysis() const
{
  int ranking[5];
  int i;
  for (i=0; i<5; i++) ranking[i]=i;
  int j;
  for (i=4; i>0; i--)
  {
    for (j=0; j<i; j++)
    {
      if (psych.data[ranking[j]] < psych.data[ranking[j+1]])
      {
        int t=ranking[j];
        ranking[j] = ranking[j+1];
        ranking[j+1] = t;
      }
    }
  }
  char  buf[128];
  wsprintf(buf, tmplt, names[ranking[0]], names[ranking[1]], names[ranking[4]]);
  return buf;
}

//-----------------------------------------------------------------------------
void FileManager::SetArchive(FRSRC* bigfile)
{
  bigfileHD = bigfile;
  voiceOverManager->SetupFiles();
}

//-----------------------------------------------------------------------------
void FileManager::ChangeCD(int newCD)
{
  assert(newCD == newDisk);
  currentDisk = newDisk;
  FMode mode = fmode;
  if (fmode==FM_PAUSED) mode=pausemode;
  assert(mode==FM_CHANGECD || mode==FM_CHANGECDR);
  if (mode == FM_CHANGECD)
  {
    AddClipsToPlay();
  }
  else if (mode==FM_CHANGECDR)
  {
    AddReminderClips();
  }
}
//-----------------------------------------------------------------------------
BOOL FileManager::DifferentDiskRetry(int& newDisk)
{
  CPNode* rtNode = playerState.GetRetryNode();
  int disk = DiskFromID(rtNode->data.film.clip);
  if (disk != currentDisk) newDisk = disk;
  return disk!=currentDisk;
}
//-----------------------------------------------------------------------------
BOOL FileManager::DifferentDiskRestart(int& newDisk)
{
  PlayerStateIterator i(playerState);
  CPNode* rsNode = i.GetNode();
  int disk = DiskFromID(rsNode->data.film.clip);
  if (disk != currentDisk) newDisk = disk;
  return disk!=currentDisk;
}
//-----------------------------------------------------------------------------
PlayerStateIterator FileManager::GetReminderStart() const
{
  PlayerStateIterator i(playerState);
  if (!globalOptions.EntirepathOn())
  {
    i.SetToEnd();
    int found=0;
    while (!i.RDone())
    {
      CPNode* n = i.GetNode();
      if (n->data.film.type == CP_LOOP)
      {
        found =1;
        break;
      }
      i.Backup();
    }
    if (!found)
    {
      i.Reset();
    }
  }
  else
  {
    i.Reset();
  }
  return i;
}
//-----------------------------------------------------------------------------
BOOL FileManager::DifferentDiskReminder(int &newDisk)
{
  PlayerStateIterator i(GetReminderStart());

  CPNode* rsNode = i.GetNode();
  int disk = DiskFromID(rsNode->data.film.clip);
  if (disk != currentDisk) newDisk = disk;
  return disk!=currentDisk;
}

//-----------------------------------------------------------------------------
BOOL FileManager::DifferentDiskResume(int& newDisk)
{
  int disk = DiskFromID(loadIt->data.film.clip);
  if (disk != currentDisk) newDisk = disk;
  return disk!=currentDisk;
}
