#include "fwl5.h"
#include "navcontr.h"
#include "linker.h"
#include "schdlr.h"
#include "resource.h"
#include "film.h"
#include "render.h"
#include "gamewin.h"
#include "dialog.h"
#include <crtdbg.h>

#include "propsh.h"
#include "optionsp.h"
#include "options.h"
#include "scorebox.h"
#include "endbox.h"
#include "navconst.h"
#include "plstate.h"
#include "splashst.h"
#include "cdstate.h"
#include "fileclip.h"
#include "prompts.h"
#include "voxover.h"
#include "options.h"
#include "dbgtst.h"
#include <stdio.h>
#include <commdlg.h>
#include <io.h>
#include <fstream.h>

NavController* NavController::it = 0; // access from static functions to single instance.
//-----------------------------------------------------------------------------
unsigned char NavConState::myOneStaticBlock[2][myBlockSize];
int NavConState::allocatedBlocks=0;
static NavConState* state = 0;
extern GameOptions globalOptions;
//-----------------------------------------------------------------------------
#ifndef NDEBUG
void* NavConState::operator new(size_t)
{
  allocatedBlocks^=1;
  return myOneStaticBlock[allocatedBlocks];
}
//-----------------------------------------------------------------------------
void NavConState::operator delete(void*)
{
}
#else
//-----------------------------------------------------------------------------
void* NavConState::operator new(size_t size)
{
  allocatedBlocks^=1;
  return size>myBlockSize?0:myOneStaticBlock[allocatedBlocks];
}
//-----------------------------------------------------------------------------
void NavConState::operator delete(void*)
{
}
#endif
//-----------------------------------------------------------------------------
void NavConState::PlayReminder()
{
}
//-----------------------------------------------------------------------------
void SetState(NavConState* s)
{
  delete state; state = s;
}
//-----------------------------------------------------------------------------
NavController::~NavController()
{
  delete scheduler;
  delete voiceOverManager;
  delete promptManager;
  delete fileManager;
  delete bigfileHD;
  delete bigfileCD;
}
//-----------------------------------------------------------------------------
NavController::NavController
  ( const char* command_
  )
    : bigfileCD(0)
    , bigfileHD(0)
    , command(command_)
    , font(IDR_FONTBITS)
    , nameOfSavedGame("SavedGame.flm")
    , startedFromIcon(FALSE)
    , schedulerOpen(FALSE)
    , fileManager(0)
    , promptManager(0)
    , voiceOverManager(0)
    , scheduler(0)
    , volume(5)
{
  it = this;
  bigfileCD = new FRSRC;
  bigfileHD = new FRSRC;
  fileManager = new FileManager;
  promptManager = new PromptManager;
  voiceOverManager = new VoiceOverManager;
  scheduler = new Scheduler;
  fileManager->TalkTo(promptManager);
  fileManager->TalkTo(this);
  fileManager->TalkTo(scheduler);
  fileManager->TalkTo(voiceOverManager);
  //fileManager->SetPath(spath);
  promptManager->SetFont(font);
  SetState(new SplashState(it));
  BigFileWaveReader::SetArchive(bigfileHD);
  fileManager->SetArchive(bigfileHD);
}
//-----------------------------------------------------------------------------
void NavController::FileManagement()
{
  fileManager->Manage();
}
//-----------------------------------------------------------------------------
void NavController::PromptManagement()
{
  promptManager->Manage();
  voiceOverManager->Manage();
}
//-----------------------------------------------------------------------------
int NavController::Idle()
{
  assert(state != 0);
  state->Idle();
  return 1;
}
//-----------------------------------------------------------------------------
IDirectSoundBuffer* psound = 0;
long originalVolume;
HRESULT hr = 0;
//#define MVRange 1200
//#define MVSteps 20
//#define MapVolume(x) \
// psound->SetVolume(-MVRange+MVRange*x/MVSteps)
//#define UnMapVolume(x) ((MVSteps*(x+MVRange)+MVRange/2)/MVRange)
const int MVRange = 1200;
const int MVSteps = 20;
inline void MapVolume(int x)
{
  hr =psound->SetVolume(-MVRange+MVRange*x/MVSteps);
}
inline int UnMapVolume(int x)
{
  return ((MVSteps*(x+MVRange)+MVRange/2)/MVRange);
}
void NavController::InitTest()
{
  DSBUFFERDESC desc;
  memset(&desc,0,sizeof(desc));
  desc.dwSize = sizeof(desc);
  desc.dwFlags = DSBCAPS_CTRLVOLUME | DSBCAPS_PRIMARYBUFFER;
  HRESULT hr = dSound->CreateSoundBuffer(&desc, &psound, 0);
  scheduler->Open(window->GetHWND(), bigfileCD);
  hr = psound->GetVolume(&originalVolume);
  //log = -MVRange+MVRange*lin/10
  //log+MVRange = MVRange*lin/10
  //10(log+MVRange)=MVRange*lin
  //10(log+MVRange)/MVRange=lin
  long dbToLin = UnMapVolume(originalVolume);

  window->IndicateVolume(dbToLin);
  schedulerOpen = TRUE;
}
//-----------------------------------------------------------------------------
void NavController::OnSize(int x, int y)
{
  //window->InvalidateAll();
  CWindowDC(window).PatBlt(CRect(0,0,x,y), BLACKNESS);
  //if (!fileManager.IsPaused())
  {
    scheduler->OnSize(x,y);
  }
}
//-----------------------------------------------------------------------------
void NavController::CloseTest()
{
  psound->Release();
  scheduler->Close();
  bigfileCD->Close();
  bigfileHD->Close();
}

//-----------------------------------------------------------------------------
void NavController::MouseDown(const CPoint& place, int flags)
{
  if (state==0) return;
  state->MouseDown(place,flags);
}

//-----------------------------------------------------------------------------
void NavController::OnPaint(const CDC& dc, const CRect& r)
{
  scheduler->OnPaint();
  if (state==0) return;
  state->OnPaint(dc, r);
}

//-----------------------------------------------------------------------------
void NavController::Key(int code, int flags)
{
  if (state==0) return;
  state->Key(code, flags);
}
//-----------------------------------------------------------------------------
void OutputLastError(const char* title)
{
  DWORD ec = GetLastError();
  char buf[256];
  FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM+255
               , 0 // lpSource
               , ec
               , 0 // languageID
               , buf
               , 255
               , 0
               );
  OutputDebugString(title);
  OutputDebugString(" says ");
  OutputDebugString(buf);
  OutputDebugString("\r\n");
}

//-----------------------------------------------------------------------------
BOOL BrowseInputFile
  ( BaseWin* owner
  , string& selected
  , string& initialDir
  , const char* type
  , const char* title
  )
{
  char szFile[270];
  OPENFILENAME ofn;
  memset(&ofn, 0, sizeof(ofn));
  strcpy(szFile, selected.c_str());
  ofn.lStructSize       = sizeof(ofn);
  ofn.hwndOwner         = owner?owner->GetHWND():0;
  ofn.hInstance         = Application::FWGetAppInstance();
  ofn.lpstrFile         = szFile;
  ofn.nMaxFile          = sizeof(szFile);
  ofn.Flags = OFN_PATHMUSTEXIST| OFN_FILEMUSTEXIST;
  ofn.nFilterIndex      = 1L;
  ofn.lpstrFilter = type;
  ofn.lpstrTitle  = title;
  ofn.lpstrInitialDir = initialDir.length()?initialDir.c_str():0;
     
  if (GetOpenFileName(&ofn))
  {
    selected = szFile;
    return TRUE;
  }
  return FALSE;
}

//-----------------------------------------------------------------------------
BOOL BrowseOutputFile
  ( BaseWin* owner
  , string& selected
  , string& initialDir
  , const char* type
  , const char* defExt
  , const char* title
  )
{
  char szFile[270];
  OPENFILENAME ofn;
  memset(&ofn, 0, sizeof(ofn));
  strcpy(szFile, selected.c_str());
  ofn.lStructSize       = sizeof(ofn);
  ofn.hwndOwner         = owner?owner->GetHWND():0;
  ofn.hInstance         = Application::FWGetAppInstance();
  ofn.lpstrFile         = szFile;
  ofn.nMaxFile          = sizeof(szFile);
  ofn.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY;
  ofn.nFilterIndex      = 1L;
  ofn.lpstrFilter = type;
  ofn.lpstrTitle  = title;
  ofn.lpstrDefExt = defExt; // this forces the extention.
  ofn.lpstrInitialDir = initialDir.length()?initialDir.c_str():0;
     
  if (GetSaveFileName(&ofn))
  {
    selected = szFile;
    return TRUE;
  }
  return FALSE;
}

void NavController::MakeSchedulerBehave()
{
  scheduler->Close();
  schedulerOpen = FALSE;
}
//-----------------------------------------------------------------------------
void NavController::ResumePlayback()
{
  window->CheckPrimary();
  //scheduler.Resume();
  fileManager->ResumePlayback();
}
//-----------------------------------------------------------------------------
void NavController::PausePlayback()
{
  fileManager->PausePlayback();
}

//-----------------------------------------------------------------------------
void NavController::Pause()
{
  if (state) state->PausePlay();
}

//-----------------------------------------------------------------------------
// Unpause:
// When the action has been paused,
// Continue (you might say 'resume') where you left off.
void NavController::Unpause()
{
  extern GameOptions globalOptions;
  window->SetWindowSize(globalOptions.ZoomOn()
                         ?GameWindow::WZ_ZOOM
                         :GameWindow::WZ_NORMAL
                        );
  if (state) state->ResumePlay();
  // This is the function called by InitialCreditState
  // to resume play after showing credits.
}

//-----------------------------------------------------------------------------
BOOL NavController::IsPaused()
{
  return state->IsPaused();
}

static char fbuf[256];
//-----------------------------------------------------------------------------
// NavController::OnWindowCreate()
// Calls various NavController methods to figure out how to start the game.
//
// cases:
// 1) No parameter:
//    Try to choose a game from registry
//    for now we can't do that.
// 2) parameter with /n
//    the other token should be a reg key.
//    Bail if it isn't
// 3) parameter alone
//    The token should be a file.
//    Bail if it isn't a valid saved game.
// Note that state->OnWindowCreate could return false if
// the dll was missing.
//
#include "dlgitem.h"
class Chooser : public Dialog
{
public:
  Chooser() : Dialog() {}
  int RunModal(BaseWin* par) { return Dialog::RunModal(par, IDD_CHOOSEGAME); }
  void OnOK(int cmd, int note);
  void OnCancel(int cmd, int note);
  void OnList(int cmd, int note);
  BOOL OnCreate();
  string chosen;
private:
  DECLARE_COMMAND_LOOKUP();
};
BEGIN_COMMAND_TABLE(Chooser)
 BIND_COMMAND(Chooser, OnOK, IDOK)
 BIND_COMMAND(Chooser, OnCancel, IDCANCEL)
 BIND_COMMAND(Chooser, OnList, IDC_GAMELIST)
END_COMMAND_TABLE();
void Chooser::OnOK(int,int)
{
  string selection;
  DialogItem list(this, IDC_GAMELIST);
  int kSel = list.SendMessage(LB_GETCURSEL, 0, 0);
  selection.resize(list.SendMessage(LB_GETTEXTLEN, kSel, 0));
  list.SendMessage(LB_GETTEXT, kSel, (LPARAM)(char*)selection.c_str());
  chosen = selection;
  Terminate(IDOK);
}
void Chooser::OnCancel(int,int)
{
  chosen = "";
  Terminate(IDCANCEL);
}
void Chooser::OnList(int,int)
{
  // could be useful.
}
BOOL Chooser::OnCreate()
{
  DialogItem list(this, IDC_GAMELIST);

  RegKey base(HKEY_CURRENT_USER, "Software\\Atari\\GameFilm");
  base.Open();
  for (int i=0; 1; i++)
  {
    string game = base.GetSubKey(i);
    if (!game.length()) break;
    list.SendMessage(LB_ADDSTRING, 0, (LPARAM)(char*)game.c_str());
  }
  return TRUE;
}
BOOL NavController::OnWindowCreate()
{
  Chooser dlg;
  const char* serr = "GameFilm startup error";
  strcpy(fbuf, command);
  char* t1 = strtok(fbuf, " ");
  char* t2; t2 = t1?strtok(0, " "):0;
  if (!t1)
  {
    Dialog::MessageBox(window, "No key or file specified", serr, MB_OK);
    if (IDOK==dlg.RunModal(window))
    {
      return (SetRegKey(dlg.chosen)==NC_SUCCESS) && state->OnWindowCreate();
    }
    else
    {
      return FALSE;
    }
  }
  if (t1 && !t2)
  {
    NCError ec = fileManager->RestoreGame(t1);
    if (NC_SUCCESS==ec)
    {
      return startedFromIcon = state->OnWindowCreate();
    }
    else
    {
      switch(ec)
      {
      case NC_NOKEY:
      case NC_NOFILE:
        strcpy(fbuf, "Invalid saved game ");
        strcat(fbuf, command);
        Dialog::MessageBox(window, fbuf, serr, MB_OK);
        break;
      case NC_REFUSED:
        break;
      }
      return FALSE;
    }
  }
  if (t1 && t2 && strcmp(t2, "/n")==0)
  {
    return (SetRegKey(t1)==NC_SUCCESS) && state->OnWindowCreate();
  }
  if (t1 && t2 && strcmp(t1, "/n")==0)
  {
    return (SetRegKey(t2)==NC_SUCCESS) && state->OnWindowCreate();
  }
  strcpy(fbuf, "We don't understand ");
  strcat(fbuf, command);
  Dialog::MessageBox(window, fbuf, serr, MB_OK);
  return FALSE;
}

//-----------------------------------------------------------------------------
// SetRegKey:
// Returns false if the string is not a valid subkey of the GameFilm key.
// Please, please, PLEASE don't browse the registry from here, this is called
// from WM_CREATE.
//
NCError NavController::SetRegKey(const string& newKey)
{
  gameKey = newKey;
  string subKey = "Software\\Atari\\GameFilm\\" + gameKey;
  globalOptions.SetRegSubKey(gameKey);
  //RegKey rootGame(HKEY_CURRENT_USER, subKey);
  if (!disks.Setup(subKey))
  {
    return NC_NOKEY;
  }

  if (!disks.RequestInsertOrQuit(window, 0))
  {
    return NC_REFUSED;
  }
  string cdPath;
  string hdPath;
  cdPath = disks[0].path;
  hdPath = disks.GetHD().path;
  if (!bigfileCD->Open(cdPath.c_str(), FRSRC_SEQ,32))
  {
    strcpy(fbuf, "Can't open CD archive at ");
    strcat(fbuf, cdPath.c_str());
    Dialog::MessageBox(window, fbuf, "GameFilm startup error", MB_OK);
    return NC_NOFILE;
  }

  if (!bigfileHD->Open(hdPath.c_str(),FRSRC_RND,32))
  {
    strcpy(fbuf, "Can't open HD archive at ");
    strcat(fbuf, hdPath.c_str());
    Dialog::MessageBox(window, fbuf, "GameFilm startup error", MB_OK);
    return NC_NOFILE;
  }
  return NC_SUCCESS;
}
//-----------------------------------------------------------------------------
void NavController::DoSaveCommand()
{
  Pause(); // must pause for this.
  window->PromptForResume();
  RegKey pathFinder(HKEY_CURRENT_USER, "Software\\Atari\\GameFilm");
  pathFinder.Open();
  string prefPath;
  if (pathFinder.Opened())
  {
    prefPath = pathFinder.GetValue("Saved");
  }
  if (BrowseOutputFile(window, nameOfSavedGame, prefPath, "GameFilms (*.flm)\0*.flm\0", "flm", "Save Game"))
  {
    fileManager->SaveGame(nameOfSavedGame.c_str());
  }
}
//-----------------------------------------------------------------------------
void NavController::DoRestoreCommand()
{
  Pause(); // must pause for this.
  window->PromptForResume();
  RegKey pathFinder(HKEY_CURRENT_USER, "Software\\Atari\\GameFilm");
  pathFinder.Open();
  string prefPath;
  if (pathFinder.Opened())
  {
    prefPath = pathFinder.GetValue("Saved");
  }
  if (BrowseInputFile(window, nameOfSavedGame, prefPath, "GameFilms (*.flm)\0*.flm\0", "Restore Game"))
  {
    if (fileManager->RestoreGame(nameOfSavedGame.c_str()))
    {
      //scheduler.Close();
      //scheduler.Open(window->GetHWND(), &bigfileCD);
      // foobar!
      //  it was WORSE to do that!
      ResumePlayback();
      window->PromptForPause();
      int newDisk;
      if (fileManager->DifferentDiskResume(newDisk))
      {
        ChangeCD(newDisk);
      }
      else
      {
        SetState(new PlayState(this));
      }
    }
  }
}

//-----------------------------------------------------------------------------
void NavController::DoOptionsCommand()
{
  Pause(); // must pause for this.
  window->PromptForResume();

  extern GameOptions globalOptions;
  PagePresentation one;
  PageParentalControl two;
  PageDifficulty three;
  int oldMenu = globalOptions.MenubarOn();
  CPropertySheet s(window, "GameFilm Options", IDI_DEFICON);
  s.Add(one);
  s.Add(two);
  s.Add(three);
  s.RunModal();
  int newMenu = globalOptions.MenubarOn();
  if (newMenu && !oldMenu)
  {
    window->ShowMenu();
  }
  else if (oldMenu && !newMenu)
  {
    window->HideMenu();
  }
}

//-----------------------------------------------------------------------------
// DoUnpauseCommand:
// When pause has been pressed, continue play.
// Could have been called 'resume.' But that word has been
// hijacked by some mental lightweight.
void NavController::DoUnpauseCommand()
{
  window->StatusMessage("");
  Unpause();
}
//-----------------------------------------------------------------------------
void NavController::DoRetryCommand()
{
  window->StatusMessage("");
  int newDisk=-1;
  if (fileManager->DifferentDiskRetry(newDisk))
  {
    ChangeCD(newDisk);
  }
  fileManager->DoRetry();
  SetState(new PlayState(this));
  window->PromptForPause();
  state->ResumePlay();
}

//-----------------------------------------------------------------------------
// DoReminderCommand:
// Perform playback of the clips shown, in the sequence
// shown, possibly with skip-ahead.
void NavController::DoReminderCommand()
{
  window->StatusMessage("REMINDER");
  window->EnableResume();
  window->PromptForPause();
  int newDisk=-1;
  if (fileManager->DifferentDiskReminder(newDisk))
  {
    ChangeCD(newDisk); // prompt for new disk
    fileManager->DoReminder(); // go ahead with reminder
    state->ResumePlay(); // do normal resumption
  }
  else
  {
    fileManager->DoReminder(); // go ahead with reminder
    SetState(new PlayState(this)); // make sure ui state is right
    state->ResumePlay(); // do normal resumption
  }
}
//-----------------------------------------------------------------------------
// DoRestartCommand:
// Start completely over from the very first clip.
void NavController::DoRestartCommand()
{
  window->StatusMessage("");
  int newDisk=-1;
  if (fileManager->DifferentDiskRestart(newDisk))
  {
    ChangeCD(newDisk);
    fileManager->StartOver();
    state->ResumePlay();
  }
  else
  {
    fileManager->StartOver();
    window->PromptForPause();
    SetState(new PlayState(this));
    state->ResumePlay();
  }
}
//-----------------------------------------------------------------------------
void NavController::DoStatusCommand(BaseWin* par)
{
  Pause(); // must pause for this.
  window->PromptForResume();
  ScoreBox sb(fileManager->GetAnalysis()
             ,fileManager->GetScore()
              ,fileManager->GetPsych()
              );
  sb.RunModal(par);
}
//-----------------------------------------------------------------------------
// DoStatusCommand:
// Show the current status of the player.
void NavController::DoStatusCommand()
{
  DoStatusCommand(window);
}
//-----------------------------------------------------------------------------
void NavController::DoGoCommand()
{
  state->GoPlay();
}

//-----------------------------------------------------------------------------
void NavController::CheckNewOrRestore()
{
  if (!schedulerOpen)
  {
    scheduler->Open(window->GetHWND(), bigfileCD);
    schedulerOpen = TRUE;
  }
  if (startedFromIcon)
  {
    ResumePlayback();
    startedFromIcon = FALSE;
    //window->PromptForPause();
  }
  else
  {
    Unpause(); //if new
  }
}
//-----------------------------------------------------------------------------
void NavController::TalkTo(GameWindow* w)
{
  window = w;
  promptManager->TalkTo(window);
}

//-----------------------------------------------------------------------------
void NavController::SizeWindow()
{
  window->SetWindowSize(globalOptions.ZoomOn()
    ?GameWindow::WZ_ZOOM
    :GameWindow::WZ_NORMAL
    );
  if (globalOptions.MenubarOn())
  {
    window->ShowMenu();
  }
}

//-----------------------------------------------------------------------------
BOOL NavController::GameOver()
{
  return fileManager->IsOver();
}

//-----------------------------------------------------------------------------
void NavController::TimeManagement()
{
  scheduler->Manager();
}

//-----------------------------------------------------------------------------
void NavController::TakeABranch()
{
  fileManager->TakeABranch();
}

//-----------------------------------------------------------------------------
BOOL NavController::WantsDialogKeys()
{
  return state->WantsDialogKeys();
}

//-----------------------------------------------------------------------------
void NavController::VolumeLouder()
{
  volume = (volume==MVSteps)?volume:(volume+1);
  window->IndicateVolume(volume);
  MapVolume(volume);
}

//-----------------------------------------------------------------------------
void NavController::VolumeSofter()
{
  volume = (volume==0)?volume:(volume-1);
  window->IndicateVolume(volume);
  MapVolume(volume);
}

//-----------------------------------------------------------------------------
void NavController::SetVolume(int vol)
{
  volume = vol;
  MapVolume(volume);
}

//-----------------------------------------------------------------------------
void NavController::ReminderEnds()
{
  window->StatusMessage("");
}

//-----------------------------------------------------------------------------
void NavController::OnAppExit()
{
  state->OnAppExit();
}

//-----------------------------------------------------------------------------
void NavController::ChangeCD(int newdisk)
{
  SetState(new CDState(this, newdisk));
  window->StatusMessage("Hiyo! CD-O!");
}

//-----------------------------------------------------------------------------
void NavController::CDInserted(int newDisk)
{
  window->StatusMessage("");
  string subKey = "Software\\Atari\\GameFilm\\" + gameKey;
  RegKey rootGame(HKEY_CURRENT_USER, subKey);
  rootGame.Open();
  char buf[20];
  sprintf(buf, "cmoo%d", newDisk);
  string cdPath = rootGame.GetValue(buf);
  scheduler->Close();//CloseArchive();
  bigfileCD->Close();
  if (!bigfileCD->Open(cdPath.c_str(), FRSRC_SEQ,32))
  {
    strcpy(fbuf, "Can't open CD archive at ");
    strcat(fbuf, cdPath.c_str());
    Dialog::MessageBox(window, fbuf, "GameFilm disk change error", MB_OK);
    window->Close();
    return; //FALSE;
  }
  scheduler->Open(window->GetHWND(), bigfileCD);//OpenArchive();
  fileManager->ChangeCD(newDisk);
  SetState(new PlayState(this));
}

//-----------------------------------------------------------------------------
BOOL NavController::PromptForCD(int index)
{
  return disks.RequestInsertOrQuit(window, index);
}

//-----------------------------------------------------------------------------
BOOL NavController::IsCDPresent(int index) const
{
  return disks.CheckPresence(index);
}

//-----------------------------------------------------------------------------
BOOL NavController::QueryClose()
{
  return state->QueryClose();
}
