#include "fwl5.h"
#include <windowsx.h>
#include "propsh.h"
#include "proppage.h"
#include "control.h"
#include <malloc.h>

BOOL CPropertySheet::Apply()
{
  return PropSheet_Apply(GetHWND());
}
BOOL CPropertySheet::CancelToClose()
{
  return PropSheet_CancelToClose(GetHWND());
}
void CPropertySheet::Changed(const CPropertyPage& page)
{
  PropSheet_Changed(GetHWND(), page.GetHWND());
}
void CPropertySheet::Unchanged(const CPropertyPage& page)
{
  PropSheet_UnChanged(GetHWND(), page.GetHWND());
}
BaseWin* CPropertySheet::GetTabControl()
{
  static Control w;
  w.SetHWND(PropSheet_GetTabControl(GetHWND()));
  return &w;
}
CPropertySheet::CPropertySheet
  ( BaseWin* parent
  , const char* title
  , const char* icon
  )
{
  Setup(parent, title, icon);
}
CPropertySheet::CPropertySheet
  ( BaseWin* parent
  , const char* title
  , int icon
  )
{
  Setup(parent, title, (const char*)icon);
}
void CPropertySheet::Setup
  ( BaseWin* parent
  , const char* title
  , const char* icon
  )
{
  memset(&rep,0,sizeof(rep));
  rep.dwSize = sizeof(rep);
  //rep.dwFlags = PSH_USECALLBACK;
  rep.hInstance = Application::FWGetAppInstance();
  if (icon)
  {
    rep.dwFlags |= PSH_USEICONID;
    rep.pszIcon = icon;
  }
  if (parent)
  {
    rep.hwndParent = parent->GetHWND();
  }
  rep.pszCaption = title;
  rep.nPages = 0;
  rep.nStartPage=0;
  rep.ppsp = 0;
  rep.phpage = 0;
  rep.pfnCallback = callback;
}
int CPropertySheet::RunModal(int first)
{
  rep.nStartPage = first;
  return PropertySheet(&rep);
}
void CPropertySheet::Select(const CPropertyPage& p)
{
  PropSheet_SetCurSel(GetHWND(), p.GetHWND(), 0);
}
int CALLBACK CPropertySheet::callback(HWND, UINT, LPARAM)
{
//  Initialize();
  return 0;
}

void CPropertySheet::Add(const CPropertyPage& p)
{
#if 0
  if (GetHWND())
  {
    PropSheet_AddPage(GetHWND(), p.GetHWND());
  }
#endif
  const_cast<CPropertyPage*>(&p)->par = this;
  int kPages = rep.nPages+1;
//  HPROPSHEETPAGE* v = (HPROPSHEETPAGE*)realloc( rep.phpage
//                                              , kPages*sizeof(HPROPSHEETPAGE)
//                                              );
  PROPSHEETPAGE* v = (PROPSHEETPAGE*)realloc( (void*)rep.ppsp
                                            , kPages*sizeof(PROPSHEETPAGE)
                                            );
  if (v)
  {
    v[kPages-1] = p.rep;//p.GetHWND();
    rep.nPages = kPages;
    rep.ppsp = v;
    rep.dwFlags |= PSH_PROPSHEETPAGE; 
    //rep.dwFlags &= ~PSH_PROPSHEETPAGE;
  }
}

CPropertySheet::~CPropertySheet()
{
  if (rep.nPages)
  {
    free(rep.phpage);
  }
}
