#include "fwl5.h"
#include <fstream.h>
#include "pstate.h"
#include "cpnodes.h"
#include "dbgnew.h"

void PlayerState::AllocateMoreNodes()
{
  nodesAllocated = nodesUsed+64;
  PlayerStateElement* newNodes;
  newNodes = (PlayerStateElement*)realloc( nodes
                                         , nodesAllocated *sizeof(PlayerStateElement)
                                         );
  nodes = newNodes;
}
void PlayerState::VisitNode(CPNode* node, const Psych& ps, long score)
{
  if (nodesUsed >= nodesAllocated)
  {
    AllocateMoreNodes();
  }
  nodes[nodesUsed].node = node;
  nodes[nodesUsed].psych = ps;
  nodes[nodesUsed].score = score;
  nodesUsed++;
}
void PlayerState::ForgetAllNodes()
{
  nodesUsed = 0;
}
BOOL Satisfied(CPNode* node)
{
  // satisfied if all links have been satisfied.
  // a link is satisfied if it is a visited film
  // or if it is a control for which all links have
  // been visited
  int j;
  int satbits =0;
  for (j=0; j<node->links; j++)
  {
    CPNode* link = node->pLinks[j];
    switch (link->nodeType)
    {
    case CPN_FILM:
      if (link->data.film.visits > 0)
      {
        satbits |= (1<<j);
      }
      break;
    case CPN_CONTROL:
      {
        for (int i=0; i<link->links; i++)
        {
          CPNode* deepLink = link->pLinks[i];
          if (deepLink->nodeType == CPN_FILM
              && deepLink->data.film.visits > 0
              )
          {
            satbits |= (1<<j);
            break;
          }
        }
      }
//      if (link->data.control.visitBits == (unsigned)((1<<link->links)-1))
//      {
//        satbits |= (1<<j);
//      }
      break;
    }
  }
  return satbits == (1<<node->links)-1;
}
//-------------------------------------------
CPNode* PlayerState::GetLastNode() const
{
  return nodes[nodesUsed-1].node;
}
//-------------------------------------------
// GetNodeAndRetry:
// This function finds the point at which a Retry should begin.
// It then flushes all nodes past the retry point.
CPNode* PlayerState::GetNodeAndRetry(Psych& newPsych, long& newScore)
{
#if 0  
  unsigned i;
  int firstLoop = 0;
  for (i=0; i<nodesUsed; i++)
  {
    if (nodes[i].node->data.film.type == CP_LOOP)
    {
      if (!Satisfied(nodes[i].node))
      {
        firstLoop = i;
        break;
      }
    }
  }
  if (i != 0)
  {
    i--; // back up one from the loop
    if (i > 0 
      && nodes[i].node->data.film.type == CP_LEADOUT 
      && nodes[i-1].node->data.film.type == CP_SHOT
      )
    {
      i--; // back up again if that was a leadout.
    }
    if (i > 0 
      && nodes[i].node->data.film.type == CP_SHOT
      && nodes[i-1].node->data.film.type == CP_LEADIN
      )
    {
      i--; // back up again if that was a shot preceded by leadin
    }
  }
#endif
  unsigned i = GetRetryNodeIndex();
  nodesUsed = i;
  newPsych = nodes[i].psych;
  newScore = nodes[i].score;
  return nodes[i].node;
}
//-------------------------------------------
static void Write(ofstream& file, unsigned v)
{
  file.write((char*)&v, sizeof(v));
}
//-------------------------------------------
static void Write(ofstream& file, const Psych& v)
{
  Write(file, v.data[0]);
  Write(file, v.data[1]);
  Write(file, v.data[2]);
  Write(file, v.data[3]);
  Write(file, v.data[4]);
}
//-------------------------------------------
BOOL PlayerState::SaveNodes(ofstream& file, unsigned baseAddress) const
{
  file.write((char*)&nodesUsed, sizeof(nodesUsed));
  for (unsigned i=0; i<nodesUsed; i++)
  {
    unsigned offset = unsigned((long)(nodes[i].node) - baseAddress);
    file.write((char*)&offset,sizeof(offset));
    file.write((char*)&nodes[i].psych, sizeof(Psych));
    file.write((char*)&nodes[i].score, sizeof(long));
  }
  return TRUE;
}
//-------------------------------------------
BOOL PlayerState::LoadNodes(ifstream& file, unsigned baseAddress)
{
  nodesUsed = 0;
  unsigned nodesToRead;
  file.read((char*)&nodesToRead, sizeof(nodesToRead));
  for (unsigned i=0; i<nodesToRead; i++)
  {
    unsigned pNodePtr;
    Psych p;
    long s;
    file.read((char*)&pNodePtr,sizeof(pNodePtr));
    file.read((char*)&p, sizeof(p));
    file.read((char*)&s, sizeof(s));
    VisitNode((CPNode*)(pNodePtr+baseAddress), p, s);
  }
  return TRUE;
}

//-------------------------------------------
unsigned PlayerState::GetRetryNodeIndex() const
{
  unsigned i=0;
  int firstLoop = 0;
#define POLICY_DONT_CHANGE_DISK_ON_RETRY
#ifdef POLICY_DONT_CHANGE_DISK_ON_RETRY
  int currentDiskMask = (nodes[nodesUsed-1].node->data.film.clip)&0xffff0000;
  for (i=nodesUsed; i>0; i--)
  {
    if ((nodes[i-1].node->data.film.clip) == currentDiskMask)
    {
      break;
    }
  }
  firstLoop = i-1;
#endif
  for (; i<nodesUsed; i++)
  {
    if (nodes[i].node->data.film.type == CP_LOOP)
    {
      if (!Satisfied(nodes[i].node))
      {
        firstLoop = i;
        break;
      }
    }
  }
  if (i != 0)
  {
    i--; // back up one from the loop
    if (i > 0 
      && nodes[i].node->data.film.type == CP_LEADOUT 
      && nodes[i-1].node->data.film.type == CP_SHOT
      )
    {
      i--; // back up again if that was a leadout.
    }
    if (i > 0 
      && nodes[i].node->data.film.type == CP_SHOT
      && nodes[i-1].node->data.film.type == CP_LEADIN
      )
    {
      i--; // back up again if that was a shot preceded by leadin
    }
  }
  return i;
}
//-------------------------------------------
// GetRetryNode:
// If you want to know ABOUT the node of retry
// before you make that the current node.
CPNode* PlayerState::GetRetryNode() const
{
  return nodes[GetRetryNodeIndex()].node;
}
