/********************************************************************\

	Output Module
	by Rudy Mathieu

\********************************************************************/

#include "RAVOut.h"
#include "Dbgtst.h"

extern HWND thisWindow;

#define SND_SIZE				4 // Must match SND_SIZE in schdlr.cpp
#define AUDIO_SKEW				4
#define BUFFERED_VIDEO_FRAMES	(SND_SIZE+AUDIO_SKEW)

#define DEFAULT_AUDIO_SIZE		918			// Default audio sample size
#define VIDEO_BUFFER_SIZE		(28 << 10)  // Max video buffer size

RAVOutput::RAVOutput()
{
	Init();
}

RAVOutput::~RAVOutput() 
{ 
	Close();
}

void RAVOutput::Init(void)
{ 
	renderer=NULL; 
	dsound=NULL; 
	sampleSize=0; 
	sample=NULL; 
	nextVideoFrame = 0;
	totalbufferedframes = 0;
	videoFrames = NULL;
	statusPAUSE = FALSE;
}

void RAVOutput::Close(void)
{
	if (videoFrames)
	{
		for (int i=0; i<BUFFERED_VIDEO_FRAMES; i++)
		{
			if (videoFrames[i])
				winFree(videoFrames[i]);
		}
		winFree(videoFrames);
	}

	winFree(sample); 
	Init(); 
}

BOOL __fastcall RAVOutput::Open(Renderer *r, DSoundBuffer *s, int ssize)
{ 
	ssize <<= 10;
	sampleSize = (ssize + 0xff) & ~0xff;
	sample =(PFrameSampleStr)winMalloc(sampleSize);
	if (sample==NULL)
	{
		Close();
		return FALSE;
	}

	videoFrames = (PVideoFrameST *) winCalloc(BUFFERED_VIDEO_FRAMES,sizeof(PVideoFrameST));
	if (videoFrames)
	{
		for (int i=0; i<BUFFERED_VIDEO_FRAMES; i++)
		{
			videoFrames[i] = (PVideoFrameST) winMalloc(VIDEO_BUFFER_SIZE);
			if (videoFrames[i]==NULL)
			{
				Close();
				return FALSE;
			}
		}
	}

	memset(sample,0,sampleSize);
	renderer=r; 
	dsound=s; 
	sampleSize=ssize;

	nextVideoFrame = 0;
	totalbufferedframes = 0;

	return TRUE;	
}

BOOL __fastcall RAVOutput::Load(FrameSampleStr *s)
{
	// Get next video buffer
	int nextbuffer = GetVideoWB();
	if (nextbuffer==-1)
	{
		// Not enough room to hold video frameso let's
		// Drop one
		DropVideoFrame(); 
		nextbuffer = GetVideoWB();
	}

	// Copy header
	memcpy(sample,s,sizeof(FrameSampleStr));

	// Copy Audio
	sample->audio = (PBYTE)sample + sizeof(FrameSampleStr);
	if (sample->audioSize)
	{
		// This RAV has audio data
		memcpy(sample->audio,(PBYTE)s + (UINT)s->audio,sample->audioSize);
	} else {
		// No audio data so just put space
		sample->audioSize = DEFAULT_AUDIO_SIZE;
		memset(sample->audio,128,DEFAULT_AUDIO_SIZE);
	}

	// Copy Video
	videoFrames[nextbuffer]->frame = sample->frame;
	sample->video =	videoFrames[nextbuffer]->data;
	memcpy(sample->video,(PBYTE)s + (UINT)s->video,sample->videoSize);
	return TRUE;
}

void RAVOutput::UpdateAV(void)
{
	if (sample==NULL || !SampleReady() || statusPAUSE==TRUE)
		return;

	if (sample->audioSize==0)
		return;
	
	// Do this only when we have audio
	if (Sound()->FillSoundBuffer(sample->audio,
   	     sample->audioSize)!=INVALID_VALUE)
	{
		if (Sound()->DonePlaying())
			Sound()->Play();

		sample->audio = NULL;
	}
}

void RAVOutput::RefreshVideo(void)
{
	if (statusPAUSE==TRUE)
	{
		area.Restore(Display()->DC());
	}
}

void RAVOutput::Pause(void) 
{ 
	statusPAUSE = TRUE; 
	Sound()->Stop();

	area.Save(Display()->DC(),NULL);
}

int	RAVOutput::GetVideoWB(void)
{
	int nb = nextVideoFrame + totalbufferedframes;
	if (nb >= BUFFERED_VIDEO_FRAMES)
		nb -= BUFFERED_VIDEO_FRAMES;

	if (totalbufferedframes >= BUFFERED_VIDEO_FRAMES)
	{
		return -1;
	}
	totalbufferedframes++;
	return nb;
}

void RAVOutput::DropVideoFrame(void)
{
	if (totalbufferedframes-- > 0)
	{
		// We are losing this frame so Fast Draw It
		Display()->FastDraw(videoFrames[nextVideoFrame]->data);
		nextVideoFrame++;
		if (nextVideoFrame >= BUFFERED_VIDEO_FRAMES)
			nextVideoFrame = 0;
	}
}
#undef SLOWIT
void RAVOutput::DisplayFrame(void)
{
	if (totalbufferedframes>0)
	{
#ifdef SLOWIT
     long i;
     for (i=0; i< 750000; i++) 
       ;
#endif
		totalbufferedframes--;
		Display()->Draw(videoFrames[nextVideoFrame]->data);
		nextVideoFrame++;
		if (nextVideoFrame >= BUFFERED_VIDEO_FRAMES)
			nextVideoFrame = 0;
#ifdef SLOWIT
     for (   ; i<1500000; i++) 
       ;
#endif

	}
}
