/********************************************************************\

  Methods:
					ResetBuffer()

			long	Read(void *ptr,long size)
			long	Seek(long)

			BOOL	Open(long size)		Size in K-bytes
					Close()

			BOOL	StatusLoaded()
			BOOL	StatusWrapped()
			BOOL	StatusFull()
			BOOL	StatusEmpty()
			BOOL	StatusStalled()
			
			LPVOID	Buffer()
			long	Size()

			long&	Start()
			long&	End()

			BOOL	DataInside(long seg1, long seg2)
			  
\********************************************************************/

#ifndef _SBUFFERHEADER
#define _SBUFFERHEADER

#include <windows.h>
#include "mem.h"


// Used by	SmartBuffer::Read()
//			RAVSampleReader::ReadSample()
//			RAVSampleReader::_Read()
#define BUFFER_EMPTY		-1
#define BUFFER_BUSY			-2
#define SAMPLE_READ_ERROR	-3

// gChunkSize is defined in SHandle.Cpp
extern long	gChunkSize;

#define BYTESLOADED() \
	((dataend>=datastart)?(dataend-datastart):(dataend-datastart+size))


class SmartBuffer {

protected:
	PBYTE	buffer;
	long	datastart,dataend;

private:
	long	size;
   long fileptr;
	long	bsize1,bsize2;
	
	BOOL	statusLOADED;
	BOOL	statusWRAPPED;
	
	void __fastcall MoveAndWrapPointer(long &data, long sizeofdata)
	{
		data += sizeofdata;
		if (data >= size) 
			data -= size;// data wrapped through top of buffer
	}

	BOOL __fastcall DataAvailable(long sizeofdata)
	{
		bsize2 = 0;
		bsize1 = sizeofdata;
		int calc = sizeofdata + datastart;
		if (calc >= size)
		{
			bsize2 = calc - size;
			bsize1 = sizeofdata - bsize2;
		}
		if (BYTESLOADED() >= sizeofdata)
			return TRUE;
		else
			return FALSE;
	}

	void	Clear(void) 
	{ 
		buffer=NULL; 
		size=0; 
		ResetBuffer();
	}

public:
	SmartBuffer() { Clear(); }
	~SmartBuffer() { CloseBuffer(); }

	void	ResetBuffer(void) 
	{ 
		bsize1 = bsize2 = 0;
		datastart=dataend=0;
		statusLOADED=statusWRAPPED=FALSE; 
		fileptr = 0;
	}
	BOOL	__fastcall Seek(long fp);
	long	__fastcall Read(void *ptr, long size);
	void	CloseBuffer(void) { winFree(buffer); Clear(); }
	BOOL	__fastcall OpenBuffer(long s);

	BOOL&	StatusLoaded(void) { return statusLOADED; }
	BOOL&	StatusWrapped(void) { return statusWRAPPED; }
	BOOL	StatusEmpty(void) { return BYTESLOADED()==0; }
	BOOL	StatusStalled(void) { return StatusFull() | StatusEmpty()| StatusLoaded(); }
	BOOL 	StatusFull(void) 
	{ 
		return (size-BYTESLOADED() < gChunkSize || 
				StatusLoaded());
	}

	PBYTE	Buffer(void) { return buffer; }
	long	Size(void) { return size; }
	long&	Start(void) { return datastart; }
	long&	End(void) { return dataend; }

	BOOL __fastcall DataInside(long seg1,long seg2)
	{
		if (BYTESLOADED()==0)
			return FALSE;

		if (dataend < datastart)
		{
			if (seg2 >= datastart || seg1 < dataend)
	  			return TRUE;
		} else {
			if (seg2 >= datastart && seg1 < dataend)
      			return TRUE;
		}
		return FALSE;
	}
};

#endif
