#include "fwl5.h"
#include "navcontr.h"
#include "splashst.h"
#include "gamewin.h"
#include "options.h"
#include "plstate.h"
#include "resource.h"
#include "wave2.h"
#include "splshdll.h"
#include "fbutton.h"
#include "icstate.h"
static int buttonLeft = 350;
static int buttonTop[3] = {100,150,200};
static BOOL (APIENTRY *pFetchBitmap)(UINT, LPBITMAPINFO *, LPVOID *, RECT *) =0;
static void (APIENTRY *pDiscardBitmap)(UINT) =0;

HBITMAP GetDLLBitmap(int i, RECT& theRect)
{
  LPBITMAPINFO header;	    // points to bitmap header
  LPVOID lpBits;		    // points to bitmap data
   if (pFetchBitmap(i, &header, &lpBits, &theRect))
   {
     HDC screenDC = GetDC(0);
   // create a DDB from the DIB and save the handle
   HBITMAP bmp = CreateDIBitmap(
                screenDC,
                (BITMAPINFOHEADER *)header,
                CBM_INIT,
                lpBits,
                header,
                (((BITMAPINFOHEADER *)header)->biClrUsed == 256) ?
	                DIB_PAL_COLORS : DIB_RGB_COLORS
           );
   ReleaseDC(0, screenDC);
   pDiscardBitmap(i);
   return bmp;
  }
   return 0;
}

//-----------------------------------------------------------------------------
  extern "C" {
  extern HANDLE _crtheap;
  };
void SplashState::GoPlay()
{
  button[0]->Destroy();
  button[1]->Destroy();
  button[2]->Destroy();
  if (hResDLL) 
  {
  	// free the bitmap resource DLL
  	FreeLibrary(hResDLL);
  }
  controller->SizeWindow();
  controller->CheckNewOrRestore();
  //SetState(new PlayState(controller));
  SetState(new InitialCreditState(controller));
}
//-----------------------------------------------------------------------------
SplashState::~SplashState()
{
  delete button[0];
  delete button[1];
  delete button[2];
}
//-----------------------------------------------------------------------------
BOOL SplashState::Setup()
{
  // find and load the bitmap resource DLL
  string subKey = "Software\\Atari\\GameFilm";
  RegKey rootGame(HKEY_CURRENT_USER, subKey);
  string dllFilename;
  rootGame.Open();
  if (rootGame.Opened())
  {
    dllFilename = rootGame.GetValue("Splash");
  }
  else
  {
    dllFilename = "splash.dll";
  }
  BOOL cool  = FALSE;
  hResDLL = LoadLibrary(dllFilename.c_str());
  if (hResDLL)
  {
    // get the entry points for the DLL functions
    pFetchBitmap = (BOOL (APIENTRY *)(UINT, LPBITMAPINFO *, LPVOID *, RECT *))
    	GetProcAddress(hResDLL, "_FetchBitmap@16");
    pDiscardBitmap = (void (APIENTRY *)(UINT))
    	GetProcAddress(hResDLL, "_DiscardBitmap@4");
    if (pFetchBitmap && pDiscardBitmap)
    {
      hbmp = GetDLLBitmap(PIC_SPLASH_SCREEN, splashRect);
      if (hbmp != 0)
      {
        button[0] = new FunkyButton(controller->window,CPoint(buttonLeft,buttonTop[0]),PIC_BTNPLAY_NORMAL, ID_HELLO_GO, 'S');
        button[1] = new FunkyButton(controller->window,CPoint(buttonLeft,buttonTop[1]),PIC_BTNOPTIONS_NORMAL, ID_HELLO_OPTIONS, 'O');
        button[2] = new FunkyButton(controller->window,CPoint(buttonLeft,buttonTop[2]),PIC_BTNEXIT_NORMAL, ID_EXITAPP, 'X');
        button[0]->SetBkgnd(hbmp);
        button[1]->SetBkgnd(hbmp);
        button[2]->SetBkgnd(hbmp);
        return TRUE;
      }
    }
    FreeLibrary(hResDLL); // could load, but not use!
  }
  string msg = "Couldn't load " + dllFilename;
  ::MessageBox(controller->window->GetHWND(), msg.c_str(), "Startup error", MB_OK); 
  return FALSE;
}
//-----------------------------------------------------------------------------
void SplashState::Idle()
{
  if (stime == 0)
  {
    stime=GetTickCount();
  }
}
//-----------------------------------------------------------------------------
void SplashState::MouseDown(const CPoint&, int flags)
{
}
//-----------------------------------------------------------------------------
void SplashState::OnAppExit()
{
  delete button[0];
  delete button[1];
  delete button[2];
}
//-----------------------------------------------------------------------------
void SplashState::OnPaint(const CDC& dc, const CRect& r)
{
  HDC codc = CreateCompatibleDC(dc.Get());
  SelectObject(codc,hbmp);
  BitBlt(dc.Get(),0,0,512,384,codc,0,0,SRCCOPY);
  DeleteDC(codc);
}


